/**
 * Copyright (C) 2003-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to 
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement 
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 *
 * @file	fs_obsolete.h
 * @brief	Header file for \ref FSOBSOLETE "Obsolete" module of Foxit PDF SDK.
 * @details	This header file contains all the deprecated macro definitions, structures and functions.<br>
 *			User is strongly recommend not to use them any more.
 */

#ifndef _FSCRT_OBSOLETE_H_
#define _FSCRT_OBSOLETE_H_

/**
 * @defgroup	FSOBSOLETE Obsolete
 * @brief		Definitions for all the deprecated macros, structures and functions of Foxit PDF SDK.<br>
 *				Definitions and functions in this module are included in fs_obsolete.h, and not recommend to use them any more.<br>
 */
/**@{*/

#ifdef __cplusplus
extern "C" {
#endif


#ifndef _FSPDF_DEF_HANDLER_ACTION_
#define _FSPDF_DEF_HANDLER_ACTION_

/** 
 * @deprecated	Current structure has been deprecated with function ::FSPDF_Doc_SetActionHandler since Foxit PDF SDK 4.5. 
 *				So, not recommend to use current structure any more. User can use structure ::FSPDF_JAVASCRIPTACTION_HANDLER instead.
 *
 * @brief	Structure for action handler information.
 *
 * @details	This is a handler and should be implemented by caller.<br>
 *			Action handler provides a number of interfaces to implement the actions.<br>
 */
typedef struct _FSPDF_ACTION_HANDLER
{
	/**
	 * @brief	User-defined data.
	 *
	 * @note	Caller can use this field to track controls.
	 */
	FS_LPVOID	clientData;

	/**
	 * @brief	Optional callback function to release data.
	 *
	 * @details	This callback function gives the implementation a chance to release any data after the interface is no longer used.
	 *			An this will be called by Foxit PDF SDK during the final cleanup process.
	 *
	 * @param[in]	clientData	Pointer to user-defined data.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*Release)(FS_LPVOID clientData);

	/**
	 * @brief	Required callback function to invalidate the client area within the specified rectangle.
	 *
	 * @details	All positions are measured in PDF "user space".
	 *			Implementation should call function ::FSPDF_RenderContext_StartPage for repainting a specified page area.
	 *
	 * @param[in]	clientData	Pointer to user-defined data.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	pageIndex	The page index which start from 0.
	 * @param[in]	pdfRect		Pointer to a <b>FSCRT_RECTF</b> structure, which specify the page area in PDF page coordinate.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 *
	 */
	FS_RESULT	(*InvalidateRect)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex, FSCRT_RECTF* pdfRect);

	/**
	 * @brief	Optional callback function to receives the current page index.
	 *
	 * @param[in]	clientData	Pointer to user-defined data.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[out]	pageIndex	Pointer to a ::FS_INT32 object that receive the index of current PDF page.
	 *							This should be stared from 0 and less than page count of current document.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetCurrentPageIndex)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32* pageIndex);
 
	/**
	 * @brief	Optional callback function to set the current page index.
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	pageIndex	Index of page index which is to be set to current PDF page, starting from 0.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*SetCurrentPageIndex)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex);

	/**
	 * @brief	Optional callback function to receive current rotation of the page view.
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	pageIndex	The page index which starts from 0.
	 * @param[out]	rotation	The rotation of the page being displayed.
	 *							Please refer to macro definitions {@link FSCRT_PAGEROTATION_0 FSCRT_PAGEROTATION_XXX} and this would be one of these macros.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetRotation)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_INT32 pageIndex, FS_INT32* rotation);
	 
	/**
	 * @brief	Optional callback function to execute a named action.
	 *
	 * @details	Please refer to description of named actions description in PDF Reference 1.7 for more details. 
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	namedAction	Pointer to a ::FSCRT_BSTR structure to specify the named action. It would be a UTF-8 string.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*ExecuteNamedAction)(FS_LPVOID clientData, FSCRT_DOCUMENT document, const FSCRT_BSTR* namedAction);

	/** 
	 * @brief	Optional callback function to set the change mark which indicates whether the content of document is changed or not.
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	changeMark	A boolean value that indicates whether the content of document is changed or not.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*SetChangeMark)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL changeMark);

	/** 
	 * @brief	Optional callback function to receive the change mark which indicates whether the content of document is changed or not.
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[out]	changeMark	Pointer to a ::FS_BOOL object that receives the change mark.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetChangeMark)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL* changeMark);
	
	/** 
	 * @brief	Optional callback function to receives handles of all documents which are opened now.
	 *
	 * @details	This function would be called by Foxit PDF SDK twice to get handles of opened document:<br>
	 *			For the first time, set parameter <i>docs</i> to <b>NULL</b> to get the actual count of opened document.
	 *			The count value will be set to parameter <i>count</i> as a result returned by first calling.
	 *			Then use <i>count</i> to allocate <i>docs</i>.<br>
	 *			For the second time, input the allocated <i>docs</i> and its size <i>count</i>. 
	 *			When this function returns successfully, <i>docs</i> will get an array of opened documents and <i>count</i> will get the actual count.<br>
	 *			The size of the input array should not be less than the actual count.<br>
	 *
	 * @param[in]		clientData	Pointer to the interface structure itself.
	 * @param[in,out]	docs		Pointer to a <b>FSCRT_DOCUMENT</b> handle that receives a array storing handles of all opened documents.
	 *								This can be <b>NULL</b> for getting actual count of opened documents before allocate this array. <br>
	 *								Please see detail description about how this callback function will be used.
	 * @param[in,out]	count		Pointer to a ::FS_INT32 object.<br>
	 *								When this function is called, it should be the size of array <i>docs</i> if the parameter <i>docs</i> is not <b>NULL</b>.<br>
	 *								When this function returns, it will receive actual count of opened documents which are got.<br>
	 *								Please see detail description about how this callback function will be used.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 * 			::FSCRT_ERRCODE_BUFFEROVERFLOW if the parameter <i>docs</i> is valid but <i>count</i> is less than required amount.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetOpenedDocs)(FS_LPVOID clientData, FSCRT_DOCUMENT* docs, FS_INT32* count);
	
	/** 
	 * @brief	Optional callback function to pop up a dialogue to show warnings or hints.
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	msg			Pointer to a ::FSCRT_BSTR structure to specify the message to be displayed. It should be a UTF-8 string.
	 * @param[in]	title		Pointer to a ::FSCRT_BSTR structure to specify the title of the dialogue. It should be a UTF-8 string.
	 * @param[in]	type		Type of button group. Its value can be:<br>
	 *							<ul>
	 *							<li>0: OK;(default value.)</li>
	 *							<li>1: OK, Cancel;</li>
	 *							<li>2: Yes, NO; </li>
	 *							<li>3: Yes, NO, Cancel.</li>
	 *							</ul>
	 * @param[in]	icon		Icon type. Its value can be:<br>
	 *							<ul>
	 *							<li>0: Error;(default value.)</li>
	 *							<li>1: Warning;</li>
	 *							<li>2: Question;</li>
	 *							<li>3: Status.</li>
	 *							</ul>
	 * @param[out]	retCode		Pointer to a ::FS_INT32 object. It can be:<br>
	 *							<ul>
	 *							<li>1: OK;</li>
	 *							<li>2: Cancel; </li>
	 *							<li>3: NO;</li>
	 *							<li>4: Yes;</li>
	 *							</ul>
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*Alert)(FS_LPVOID clientData, const FSCRT_BSTR* msg, const FSCRT_BSTR* title, FS_INT32 type, FS_INT32 icon, FS_INT32* retCode);
	
	/** 
	 * @brief	Optional callback function to cause the system to play a sound. 
	 *
	 * @param[in]	clientData	Pointer to the interface structure itself.
	 * @param[in]	type		Sound type. Its value can be:<br>
	 *							<ul>
	 *							<li>0: Error</li>
	 *							<li>1: Warning</li>
	 *							<li>2: Question</li>
	 *							<li>3: Status</li>
	 *							<li>4: Default (default value)</li>
	 *							</ul>
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*Beep)(FS_LPVOID clientData, FS_INT32 type);
	
	/** 
	 * @brief	Optional callback function to display a dialogue box containing a question and an entry field for the user to reply to the question.  
	 *
	 * @param[in]	clientData		The user-supplied data.
	 * @param[in]	question		Pointer to a ::FSCRT_BSTR structure to specify the question to be posed to the user. It should be a UTF-8 string.
	 * @param[in]	title			Pointer to a ::FSCRT_BSTR structure to specify the title of the dialogue box. It should be a UTF-8 string.
	 * @param[in]	defaultValue	Pointer to a ::FSCRT_BSTR structure to specify a default value for the answer to the question. It should be a UTF-8 string.
	 *								If the answer is not specified, no default value is presented.
	 * @param[in]	label			Pointer to a ::FSCRT_BSTR structure to specify a short string to appear in front of the edit text field. It should be a UTF-8 string.
	 * @param[in]	isPassword		A boolean value that indicates whether user's response should be masked to show:<br>
	 *								<b>TRUE</b> means that the user's response should show as asterisks (*) or bullets (?) 
	 *								to mask the response, which might be sensitive information. <br>
	 *								Default: <b>FALSE</b>.
	 * @param[out]	response		Pointer to a ::FSCRT_BSTR structure that receives the user's response. It can be <b>NULL</b>.
	 *								And it should be a UTF-8 string if it is not <b>NULL</b>.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*Response)(FS_LPVOID clientData, const FSCRT_BSTR* question, const FSCRT_BSTR* title, const FSCRT_BSTR* defaultValue, 
							const FSCRT_BSTR* label, FS_BOOL isPassword, FSCRT_BSTR* response);
	
	/**
	 * @brief	Optional callback function to get the file path of current document. 
	 *
	 * @param[in]	clientData	The user-supplied data.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[out]	filePath	Pointer to a ::FSCRT_BSTR structure that receives the file path. It can be <b>NULL</b>
	 *							And it should be a UTF-8 string if it is not <b>NULL</b>.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetFilePath)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FSCRT_BSTR* filePath);
	
	/**
	 * @brief	Optional callback function to mail current PDF document as an attachment to all recipients, with or without user interaction. 
	 *
	 * @param[in]	clientData	The user-supplied data.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	bUI			A boolean value:<br>
	 *							<b>TRUE</b> means that the rest of the parameters are used in a compose-new-message window 
	 *							which is displayed to the user.<br>
	 *							<b>FALSE</b> means that the parameter <i>To</i> is required and all others are optional.
	 * @param[in]	To			Pointer to a ::FSCRT_BSTR structure to specify a semicolon-delimited list of recipients for the message. It should be a UTF-8 string.
	 * @param[in]	Subject		Pointer to a ::FSCRT_BSTR structure to specify the subject of the message. It should be a UTF-8 string.<br>
	 *							The limit of its length is 64 KB.
	 * @param[in]	CC			Pointer to a ::FSCRT_BSTR structure to specify a semicolon-delimited list of CC recipients for the message. It should be a UTF-8 string.
	 * @param[in]	BCC			Pointer to a ::FSCRT_BSTR structure to specify a semicolon-delimited list of BCC recipients for the message. It should be a UTF-8 string.
	 * @param[in]	Msg			Pointer to a ::FSCRT_BSTR structure to specify the content of the message. It should be a UTF-8 string.<br>
	 *							The limit of its length is 64 KB.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*Mail)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL bUI, const FSCRT_BSTR* To, 
					    const FSCRT_BSTR* Subject, const FSCRT_BSTR* CC, const FSCRT_BSTR* BCC, const FSCRT_BSTR* Msg); 

	/**
	 * @brief	Optional callback function to print all or a specific number of pages of the document.
	 *
	 * @param[in]	clientData		The user-supplied data.
	 * @param[in]	document		Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	isUI			<b>TRUE</b> means that a UI will be presented to the user to obtain printing information and confirm the action.
	 * @param[in]	start			A zero-based index of the start of an inclusive range of pages.
	 * @param[in]	end				A zero-based index of the end of an inclusive page range.
	 * @param[in]	isSilent		<b>TRUE</b> means that the cancel dialogue box will be suppressed while the document is printing. 
	 *								The default is <b>FALSE</b>.
	 * @param[in]	isShrinkToFit	<b>TRUE</b> means that the page is shrunk (if necessary) to fit within the area of the printed page.
	 * @param[in]	isPrintAsImage	<b>TRUE</b> means that it will print pages as an image.
	 * @param[in]	isReverse		<b>TRUE</b> means that it will print from <i>nEnd</i> to <i>nStart</i>.
	 * @param[in]	isAnnotations	<b>TRUE</b> means that annotations will be printed.The default is <b>TRUE</b>.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*Print)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_BOOL isUI, FS_INT32 start, FS_INT32 end, FS_BOOL isSilent ,
					     FS_BOOL isShrinkToFit, FS_BOOL isPrintAsImage, FS_BOOL isReverse, FS_BOOL isAnnotations);

	/**
	 * @brief	Optional callback function to submit the form data to a specified URL.
	 *
	 * @param[in]	clientData	The user-supplied data.
	 * @param[in]	document	Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
	 * @param[in]	formData	Pointer to a data buffer which is to be submit.
	 * @param[in]	length		The size(in bytes) of the buffer parameter<i>formData</i>.
	 * @param[in]	URL			Pointer to a ::FSCRT_BSTR structure to specify a URL, to which the form data will be submit. It should be a UTF-8 string.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*SubmitForm)(FS_LPVOID clientData, FSCRT_DOCUMENT document, FS_LPVOID formData, FS_DWORD length, const FSCRT_BSTR* URL);
	
	/**
	 * @brief	Optional callback function to launch to a specified URL.
	 *
	 * @param[in]	clientData	The user-supplied data.
	 * @param[in]	URL			Pointer to a ::FSCRT_BSTR structure to specify a URL. It should be a UTF-8 string.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*LaunchURL)(FS_LPVOID clientData, const FSCRT_BSTR* URL); 

	/**
	 * @brief	Optional callback function to show a file selection dialogue, and return the selected file path.
	 *
	 * @param[in]	clientData	The user-supplied data.
	 * @param[out]	filePath	Pointer to a ::FSCRT_BSTR structure that receives the file path. It can be <b>NULL</b>
	 *							And it should be a UTF-8 string if it is not <b>NULL</b>.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 *
	 * @note		The <i>filePath</i> should be always inputted in the local encoding.
	 */
	FS_RESULT	(*BrowseFile)(FS_LPVOID clientData, FSCRT_BSTR* filePath);

	/**
	 * @brief	Optional callback function to get the language of the running viewer application.
	 *
	 * @param[in]	clientData	The user-supplied data.
	 * @param[out]	language	Pointer to a ::FS_INT32 object that receives the language.
	 *							Please refer to macro definitions {@link FSCRT_LANGUAGE_UNK FSCRT_LANGUAGE_XXX} and this would be one of these macros.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetLanguage)(FS_LPVOID clientData, FS_INT32* language);

	/*
	 * @brief	Optional callback function to get identity properties of the current user.
	 *
	 * @param[in]	clientData			The user-supplied data.
	 * @param[out]	identityProperties	Pointer to a ::FSPDF_IDENTITYPROPERTIES structure that receives the identity properties.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*GetIdentityProperties)(FS_LPVOID clientData, FSPDF_IDENTITYPROPERTIES* identityProperties);

	/**
	 * @brief	Optional callback function to pop up a menu window.
	 * 
	 * @details	Application should call function ::FSCRT_BStr_Set to set the value of <i>seletedMenu</i>, and do not call function ::FSCRT_BStr_Clear to free <i>seletedMenu</i>.
	 *			The position of menu window is same as cursor's. 
	 *
	 * @param[in]	clientData		The user-supplied data.
	 * @param[in]	menus			Pointer to an array of ::FSPDF_MENULIST object which is the menus.
	 * @param[in]	count			The size of <i>menus</i>.
	 * @param[out]	seletedMenu		Pointer to a ::FSCRT_BSTR structure that receives selected menu.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*PopupMenu)(FS_LPVOID clientData, const FSPDF_MENULIST* menus, FS_INT32 count, FSCRT_BSTR* seletedMenu);
} FSPDF_ACTION_HANDLER;

#endif /* _FSPDF_DEF_HANDLER_ACTION_ */

/**
 * @deprecated	Current function has been deprecated since Foxit PDF SDK 4.5. So, not recommend to use current function any more. User can use function ::FSPDF_Doc_SetJavaScriptActionHandler instead.
 *
 * @brief	Set the action handler to the PDF document.
 *
 * @details	Some interaction features such as performing JavaScript actions or filling the form, would require caller to implement the callback functions of action handler,
 *			in order to function well.	
 *
 * @param[in]	document			Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
 * @param[in]	actionHandler		Pointer to a ::FSPDF_ACTION_HANDLER structure.
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_PARAM if the parameter <i>document</i> or <i>actionHandler</i> is a <b>NULL</b> pointer.<br>
 * 			::FSCRT_ERRCODE_INVALIDTYPE if the parameter <i>document</i> is not a valid PDF document type.<br>
 * 			::FSCRT_ERRCODE_HANDLER if FSPDF_ACTION_HANDLER::InvalidateRect of parameter <i>actionHandler</i> is <b>NULL</b><br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.<br>
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT	FSPDF_Doc_SetActionHandler(FSCRT_DOCUMENT document, FSPDF_ACTION_HANDLER* actionHandler);

#ifndef _FSPDF_DEF_STRUCTURE_ENUMPAGESIZEHANDLER_
#define _FSPDF_DEF_STRUCTURE_ENUMPAGESIZEHANDLER_

/** 
 * @deprecated	Current structure has been deprecated with function :: FSPDF_Doc_EnumAllPageSize, since Foxit PDF SDK 4.5. 
 *				So, not recommend to use current structure any more. User can use structure ::FSPDF_ENUMPAGEINFOHANDLER instead.
 *
 * @brief	Structure for page size enumeration.
 *
 * @note	This is a handler and should be implemented by caller.<br>
 *			This handler provides method to allow application to enumerate page size.<br>
 */
typedef struct _FSPDF_ENUMPAGESIZEHANDLER
{
	/**
	 * @brief	User-defined data.
	 *
	 * @note	Caller can use this field to track controls.
	 */
	FS_LPVOID	clientData;

	/*
	 * @brief	Enumerate page size of a PDF page.
	 *
	 * @details	Foxit SDK calls this function to enumerate page size for each page, and the application can get page size quickly 
	 *			from the parameter <i>pageWidth</i> and <i>pageHeight</i>, with <i>pageIndex</i> in this callback function.<br>
	 *			Application is not suggested to call any API of Foxit SDK inside this callback function since it may reduce the speed of enumerating all page size.
	 *
	 * @param[in]	clientData		Pointer to user-defined data.
	 * @param[in]	pageIndex		Index of a PDF page. starting from 0.
	 * @param[in]	pageWidth		Width of a PDF page.
	 * @param[in]	pageHeight		Height of a PDF page.
	 *
	 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
	 *			::FSCRT_ERRCODE_UNRECOVERABLE if function finds an unrecoverable error internally.<br>
	 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
	 */
	FS_RESULT	(*EnumPageSize)(FS_LPVOID clientData, FS_INT32 pageIndex, FS_FLOAT pageWidth, FS_FLOAT pageHeight);
}FSPDF_ENUMPAGESIZEHANDLER;

#endif //_FSPDF_DEF_STRUCTURE_ENUMPAGESIZEHANDLER_

/**
 * @deprecated	Current function has been deprecated since Foxit PDF SDK 4.5. So, not recommend to use current function any more. User can use function ::FSPDF_Doc_EnumPagesInfo instead.
 *
 * @brief	Enumerate the size of all page.
 *
 * @details	This function is used for enumerating all page's size quickly. Application can get page size for each page from callback function FSPDF_ENUMPAGESIZEHANDLER::EnumPageSize.<br>
 *			if the parameter <i>document</i> is loaded in asynchronous mode, application should ensure that data of <i>document</i> is available, and the data of all pages are available as well.
 *
 * @param[in]	document		Handle to a <b>FSCRT_DOCUMENT</b> object which is a PDF document object. It should be valid.
 * @param[in]	enumHandler		Pointer to a ::FSPDF_ENUMPAGESIZEHANDLER structure which is a page size enumeration handler.
 *
 * @return	::FSCRT_ERRCODE_SUCCESS for success.<br>
 *			::FSCRT_ERRCODE_DATANOTREADY if the data of parameter <i>document</i> is not available, or page data in PDF file is not available.<br>
 *			::FSCRT_ERRCODE_PARAM if the parameter <i>document</i> <i>enumHandler</i> or FSPDF_ENUMPAGESIZEHANDLER::EnumPageSize of <i>enumHandler</i> is <b>NULL</b>.<br>
 * 			::FSCRT_ERRCODE_INVALIDTYPE if the parameter <i>document</i> is not a valid PDF document type.<br>
 * 			::FSCRT_ERRCODE_UNRECOVERABLE if the function finds an unrecoverable error internally.<br>
 *			::FSCRT_ERRCODE_DATANOTREADY if there are not available data for ASYNC documents.<br>
 *			::FSCRT_ERRCODE_ERROR if failing to enumerate sizes of all pages because of any other reason.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.<br>
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>document</i>: this handle is long-term partially recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT FSPDF_Doc_EnumAllPageSize(FSCRT_DOCUMENT document, FSPDF_ENUMPAGESIZEHANDLER* enumHandler);

#ifndef _FSPDF_DEF_MACRO_TEXTDIRECTION_
#define _FSPDF_DEF_MACRO_TEXTDIRECTION_
/** 
 * @deprecated	Current macro definitions have been deprecated with function ::FSPDF_TextPage_GetNextCharIndexByDirection, since Foxit PDF SDK 4.3. 
 *				So, not recommend to use current function any more.
 * @name	Macro Definitions for Text Direction Flags
 * @note	These are used in function ::FSPDF_TextPage_GetNextCharIndexByDirection.
 */
/**@{*/

/** @brief	Text direction: left. */
#define FSPDF_TEXTDIRECTION_LEFT		-1
/** @brief	Text direction: right. */
#define FSPDF_TEXTDIRECTION_RIGHT		1
/** @brief	Text direction: up. */
#define FSPDF_TEXTDIRECTION_UP			-2
/** @brief	Text direction: down. */
#define FSPDF_TEXTDIRECTION_DOWN		2

/**@}*/
#endif /* _FSPDF_DEF_MACRO_TEXTDIRECTION_ */

/**
 * @deprecated	Current function has been deprecated since Foxit PDF SDK 4.3. So, not recommend to use current function any more.
 *
 * @brief	Get index of next character of a specific character in a specific direction.
 *
 * @param[in]	textPage		Handle to a <b>FSPDF_TEXTPAGE</b> object returned by function ::FSPDF_TextPage_Load.
 * @param[in]	curIndex		A zero-based index for current character. 
 *								Range: from 0 to (charcount - 1).<i>charcount</i> is returned by function ::FSPDF_TextPage_CountChars.
 * @param[in]	direction		Indicates the direction to get the next character.
 *								Please refer to macro definitions {@link FSPDF_TEXTDIRECTION_LEFT FSPDF_TEXTDIRECTION_XXX} and this should be one of these macros.
 * @param[out]	nextIndex		Pointer to a ::FS_INT32 object that receives a zero-based index for the next character. <br>
 *								The meaning of some special return values for errors are:<br>
 *								<ul>
 *								<li>-1 : reach the beginning of the page</li> 
 *								<li>-2 : reach the end of the page</li>
 *								<li>-3 : other error</li>
 *								</ul>
 *
 * @return	::FSCRT_ERRCODE_SUCCESS if successful.<br>
 *			::FSCRT_ERRCODE_PARAM if parameter <i>textPage</i> or <i>nextIndex</i> is a <b>NULL</b> pointer, or <i>direction</i> is invalid.<br>
 * 			::FSCRT_ERRCODE_NOTFOUND if parameter <i>curIndex</i> is out of range, or no next character is found.<br>
 *			::FSCRT_ERRCODE_UNRECOVERABLE if the function can not be recovered.<br>
 *			::FSCRT_ERRCODE_ERROR if the function cannot get the index of character by direction because of any other reason.<br>
 *			For more error codes, please refer to macro definitions {@link FSCRT_ERRCODE_SUCCESS FSCRT_ERRCODE_XXX}.
 *
 * @attention	<b>Thread Safety</b>: this function is thread safe.<br>
 *				<b>OOM Information</b>:<br>
 *				OOM handling is only for mobile platforms, not for server or desktop.<br>
 *				<ul>
 *				<li>This function is long-term unrecoverable.</li>
 *				<li> <i>textPage</i>: this handle is long-term partially recoverable.</li>
 *				</ul>
 *				Please refer to the document "Robust PDF Applications with Limited Memory" for more details.
 */
FS_RESULT 	FSPDF_TextPage_GetNextCharIndexByDirection(FSPDF_TEXTPAGE textPage, FS_INT32 curIndex, FS_INT32 direction, FS_INT32* nextIndex);

#ifndef FSPDF_RENDERCONTEXTFLAG_LIMITEDIMAGECACHE
/** 
 * @deprecated	Current macro definition has been deprecated , since Foxit PDF SDK 5.1. 
 *				So, not recommend to use it to function ::FSPDF_RenderContext_SetFlags any more.
 *
 * @brief	Limit image cache size. 
 *
 * @details	This will help to save memory for a large amount of image data. 
 */
#define FSPDF_RENDERCONTEXTFLAG_LIMITEDIMAGECACHE		0x0010
#endif

#ifndef FSPDF_DEF_MACRO_ANNOT_ICONNAME
/** 
 * @deprecated	Current macro definition has been deprecated, since Foxit PDF SDK 5.2. 
 *				So, not recommend to use it any more. User can use macro definition ::FSPDF_ANNOT_ICONNAME_TEXT_CHECK instead.
 *
 * @brief	Note icon type: Checkmark.
 */
#define FSPDF_ANNOT_ICONNAME_TEXT_CHECKMARK				"Checkmark"
#endif //FSPDF_DEF_MACRO_ANNOT_ICONNAME]

#ifdef __cplusplus
};
#endif

/**@}*/ /* group FSAPP */

#endif /* _FSCRT_APP_R_H_ */

