﻿//
// renderPage.xaml.cpp
// Implementation of the renderPage class
//

#include "pch.h"
#include <iostream>
#include "renderPage.xaml.h"

using namespace demo_view;

using namespace Platform;
using namespace Platform::Collections;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;
using namespace Windows::Graphics::Display;
using namespace Windows::UI::Xaml;
using namespace Windows::UI::Xaml::Controls;
using namespace Windows::UI::Xaml::Controls::Primitives;
using namespace Windows::UI::Xaml::Data;
using namespace Windows::UI::Xaml::Input;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::UI::Xaml::Media;
using namespace Windows::UI::Xaml::Navigation;

using namespace concurrency;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::UI::Xaml::Media::Imaging;
using namespace Windows::UI::Xaml::Interop;
using namespace Windows::ApplicationModel;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Storage::Streams;
using namespace Windows::Graphics::Imaging;
using namespace Windows::Storage::Pickers;
using namespace Windows::ApplicationModel::Activation;
using namespace Windows::UI::Popups;

#if !defined (_DEMOVIEW_) && !defined(_DEMOANNOT_)
using namespace SDKDemoCommon;
#endif

// The Basic Page item template is documented at http://go.microsoft.com/fwlink/?LinkID=390556

renderPage^ renderPage::Current = nullptr;

renderPage::renderPage()
{
	InitializeComponent();
	SetValue(_defaultViewModelProperty, ref new Platform::Collections::Map<String^, Object^>(std::less<String^>()));
	auto navigationHelper = ref new Common::NavigationHelper(this);
	SetValue(_navigationHelperProperty, navigationHelper);
	navigationHelper->LoadState += ref new Common::LoadStateEventHandler(this, &renderPage::LoadState);
	navigationHelper->SaveState += ref new Common::SaveStateEventHandler(this, &renderPage::SaveState);

	m_bReleaseLibrary = false;
	m_SDKDocument = nullptr;
	//Initialize, otherwise no method of SDK can be used.
	FS_RESULT iRet = FSDK_Initialize();
	if (FSCRT_ERRCODE_SUCCESS != iRet)
	{
		String ^errMsg = "Error: Fail to initialize SDK library.";
		if (FSCRT_ERRCODE_INVALIDLICENSE == iRet)
			errMsg += "License is invalid or expired!!!";
		ShowErrorLog(errMsg, ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler),
					 true, iRet);
		return;
	}
	m_bReleaseLibrary = true;

	m_SDKDocument = ref new FSDK_Document();
	m_PDFDoc = NULL;
	m_iCurPageIndex = 0;
	m_PDFPage = NULL;
	m_fPageWidth = 0.0f;
	m_fPageHeight = 0.0f;

	m_iStartX = 0;
	m_iStartY = 0;
	m_iRenderAreaSizeX = 0;
	m_iRenderAreaSizeY = 0;
	m_iRotation = FSCRT_PAGEROTATION_0;

	m_dbScaleDelta = 0.05f;
	m_dbScaleFator = 1.0f;
	m_dbCommonFitWidthScale = 1.0f;
	m_dbCommonFitHeightScale = 1.0f;
	m_dbRotateFitWidthScale = 1.0f;
	m_dbRotateFitHeightScale = 1.0f;

	m_bFitWidth = false;
	m_bFitHeight = false;

	renderPage::Current = this;
	//PickPDFFile();
}

renderPage::~renderPage()
{
}

void renderPage::ReleaseResources()
{
	if (m_SDKDocument)
		m_SDKDocument->ReleaseResource();
	FSDK_Finalize();
	m_bReleaseLibrary = false;
}

DependencyProperty^ renderPage::_defaultViewModelProperty =
DependencyProperty::Register("DefaultViewModel",
TypeName(IObservableMap<String^, Object^>::typeid), TypeName(renderPage::typeid), nullptr);

/// <summary>
/// Used as a trivial view model.
/// </summary>
IObservableMap<String^, Object^>^ renderPage::DefaultViewModel::get()
{
	return safe_cast<IObservableMap<String^, Object^>^>(GetValue(_defaultViewModelProperty));
}

DependencyProperty^ renderPage::_navigationHelperProperty =
DependencyProperty::Register("NavigationHelper",
TypeName(Common::NavigationHelper::typeid), TypeName(renderPage::typeid), nullptr);

/// <summary>
/// Gets an implementation of <see cref="NavigationHelper"/> designed to be
/// used as a trivial view model.
/// </summary>
Common::NavigationHelper^ renderPage::NavigationHelper::get()
{
	return safe_cast<Common::NavigationHelper^>(GetValue(_navigationHelperProperty));
}

#pragma region Navigation support

/// The methods provided in this section are simply used to allow
/// NavigationHelper to respond to the page's navigation methods.
/// 
/// Page specific logic should be placed in event handlers for the  
/// <see cref="NavigationHelper::LoadState"/>
/// and <see cref="NavigationHelper::SaveState"/>.
/// The navigation parameter is available in the LoadState method 
/// in addition to page state preserved during an earlier session.

void renderPage::OnNavigatedTo(NavigationEventArgs^ e)
{
	//NavigationHelper->OnNavigatedTo(e);
	m_pPDFFile = safe_cast<StorageFile^>(e->Parameter);
	if (m_pPDFFile)
	{
		OpenPdfDocument(m_pPDFFile);
	}
}

void renderPage::OnNavigatedFrom(NavigationEventArgs^ e)
{
	// When trigger back button in hardware, it means to go back to last page -- so, release resources here.
	if (true == m_bReleaseLibrary)
		ReleaseResources();

	NavigationHelper->OnNavigatedFrom(e);
}

#pragma endregion

/// <summary>
/// Populates the page with content passed during navigation. Any saved state is also
/// provided when recreating a page from a prior session.
/// </summary>
/// <param name="sender">
/// The source of the event; typically <see cref="NavigationHelper"/>
/// </param>
/// <param name="e">Event data that provides both the navigation parameter passed to
/// <see cref="Frame::Navigate(Type, Object)"/> when this page was initially requested and
/// a dictionary of state preserved by this page during an earlier
/// session. The state will be null the first time a page is visited.</param>
void renderPage::LoadState(Object^ sender, Common::LoadStateEventArgs^ e)
{
	(void)sender;	// Unused parameter
	(void)e;	// Unused parameter
}

/// <summary>
/// Preserves state associated with this page in case the application is suspended or the
/// page is discarded from the navigation cache.  Values must conform to the serialization
/// requirements of <see cref="SuspensionManager::SessionState"/>.
/// </summary>
/// <param name="sender">The source of the event; typically <see cref="NavigationHelper"/></param>
/// <param name="e">Event data that provides an empty dictionary to be populated with
/// serializable state.</param>
void renderPage::SaveState(Object^ sender, Common::SaveStateEventArgs^ e){
	(void)sender;	// Unused parameter
	(void)e; // Unused parameter
}

void renderPage::PickPDFFile()
{
	FileOpenPicker^ openPicker = ref new FileOpenPicker();
	openPicker->ViewMode = PickerViewMode::List;
	openPicker->SuggestedStartLocation = PickerLocationId::Desktop;
	openPicker->FileTypeFilter->Append(".pdf");

	openPicker->PickSingleFileAndContinue();
}

void renderPage::OpenPdfDocument(Windows::Storage::StorageFile^ file)
{
	if (file != nullptr)
	{
		IAsyncOperation<BasicProperties ^>^ file_property = file->GetBasicPropertiesAsync();
		create_task(file_property).then([=](BasicProperties^ properties)
		{
			m_SDKDocument = ref new FSDK_Document();

			//Load PDF document
			create_task(m_SDKDocument->OpenDocumentAsync(file, (int)(properties->Size), nullptr)).then([=](FS_RESULT ret) {
				if (FSCRT_ERRCODE_SUCCESS != ret)
				{
					ShowErrorLog("Error: Fail to load PDF document.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler),
						true, ret);
					return;
				}
				m_PDFDoc = (FSCRT_DOCUMENT)m_SDKDocument->m_hDoc.pointer;
				//Load PDF page
				ret = LoadPage(m_iCurPageIndex);
				if (FSCRT_ERRCODE_SUCCESS != ret)
					return;
				//Show PDF page to device.
				m_dbScaleFator = (m_dbCommonFitWidthScale < m_dbCommonFitHeightScale) ? m_dbCommonFitWidthScale : m_dbCommonFitHeightScale;
				ShowPage();
			});
		});
	}
	else
	{
		ShowErrorLog("Error: Fail to pick a valid PDF file.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
}

void renderPage::ContinueFileOpenPicker(FileOpenPickerContinuationEventArgs^ args)
{//This function will be called by App::OnActivated if event kind is ActivationKind::PickFileContinuation.
	StorageFile^ file;
	if (args->Files->Size > 0)
	{
		// Open a stream for the selected file.
		file = args->Files->GetAt(0);
	}
	else
	{
		ShowErrorLog("Error: Fail to pick a PDF file to open.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}

	if (file != nullptr)
	{
		IAsyncOperation<BasicProperties ^>^ file_property = file->GetBasicPropertiesAsync();
		create_task(file_property).then([=](BasicProperties^ properties)
		{
			m_SDKDocument = ref new FSDK_Document();
			
			//Load PDF document
			create_task(m_SDKDocument->OpenDocumentAsync(file, (int)(properties->Size), nullptr)).then([=](FS_RESULT ret){
				if (FSCRT_ERRCODE_SUCCESS != ret)
				{
					ShowErrorLog("Error: Fail to load PDF document.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler),
						true, ret);
					return;
				}
				m_PDFDoc = (FSCRT_DOCUMENT)m_SDKDocument->m_hDoc.pointer;
				//Load PDF page
				ret = LoadPage(m_iCurPageIndex);
				if (FSCRT_ERRCODE_SUCCESS != ret)
					return;
				//Show PDF page to device.
				m_dbScaleFator = (m_dbCommonFitWidthScale < m_dbCommonFitHeightScale) ? m_dbCommonFitWidthScale : m_dbCommonFitHeightScale;
				ShowPage();
			});
		});
	}
	else
	{
		ShowErrorLog("Error: Fail to pick a valid PDF file.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
}

FS_RESULT renderPage::LoadPage(int iPageIndex)
{// To load and parse page.
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	if (NULL == m_PDFDoc)
	{
		ShowErrorLog("Error: No PDF document has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return ret;
	}

	if (m_PDFPage)
	{
		FSPDF_Page_Clear(m_PDFPage);
		m_PDFPage = NULL;
	}
	ret = m_SDKDocument->LoadPageSync(iPageIndex);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		m_PDFPage = (FSCRT_PAGE)m_SDKDocument->m_hPage.pointer;
		m_iCurPageIndex = iPageIndex;
		GetPageInfo();
	}
	else
		ShowErrorLog("Error: Fail to load and parse a page.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler), 
					 true, ret);

	return ret;
}

void renderPage::GetPageInfo()
{// To get page size and prepare some necessary data.
	if (NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
	FS_RESULT ret = FSPDF_Page_GetSize(m_PDFPage, (FS_FLOAT*)&m_fPageWidth, (FS_FLOAT*)&m_fPageHeight);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		double scrollViewerW = scroll_pdf->ActualWidth;
		double scrollViewerH = scroll_pdf->ActualHeight;

		m_dbCommonFitWidthScale = scrollViewerW / m_fPageWidth;
		m_dbCommonFitHeightScale = scrollViewerH / m_fPageHeight;

		m_dbRotateFitWidthScale = scrollViewerW / m_fPageHeight;
		m_dbRotateFitHeightScale = scrollViewerH / m_fPageWidth;
	}
}

void renderPage::ShowPage()
{//To render PDF page, finally to the image control.	
	//Calculate render size.
	CalcRenderSize();
	PixelSource^ bitmap = ref new PixelSource();
	bitmap->Width = (int)m_iRenderAreaSizeX;
	bitmap->Height = (int)m_iRenderAreaSizeY;
	task<IRandomAccessStreamWithContentType^>(m_SDKDocument->RenderPageAsync(bitmap, m_iStartX, m_iStartY, m_iRenderAreaSizeX, m_iRenderAreaSizeY, m_iRotation)).then(
		[=](IRandomAccessStreamWithContentType^ stream){
		if (stream == nullptr)
		{
			ShowErrorLog("Error: Fail to render page.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler),
				true, FSCRT_ERRCODE_ERROR);
			return;
		}
		BitmapImage^ bmpImage = ref new BitmapImage;
		bmpImage->SetSource(stream);
		image->Width = (int)m_iRenderAreaSizeX;
		image->Height = (int)m_iRenderAreaSizeY;
		image->Source = bmpImage;
	});
}

void renderPage::Click_BTN_NextPage(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to turn to the next page
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
	FS_INT32 iPageCount;
	FSPDF_Doc_CountPages(m_PDFDoc, &iPageCount);
	FS_INT32 iPageIndex = m_iCurPageIndex;
	if (iPageIndex < iPageCount - 1)
	{
		iPageIndex++;
		FS_RESULT ret = LoadPage(iPageIndex);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			return;
		}
		ShowPage();
	}
}

void renderPage::Click_BTN_PrePage(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to turn to the previous page
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
	FS_INT32 iPageCount;
	FSPDF_Doc_CountPages(m_PDFDoc, &iPageCount);
	FS_INT32 iPageIndex = m_iCurPageIndex;

	if (iPageIndex > 0)
	{
		iPageIndex--;
		FS_RESULT ret = LoadPage(iPageIndex);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			return;
		}
		ShowPage();
	}
}

void renderPage::CalcRenderSize()
{// To calculate render size.
	if (FSCRT_PAGEROTATION_90 == m_iRotation || FSCRT_PAGEROTATION_270 == m_iRotation)
	{
		if (m_bFitWidth)
			m_dbScaleFator = m_dbRotateFitWidthScale;
		if (m_bFitHeight)
			m_dbScaleFator = m_dbRotateFitHeightScale;

		m_iRenderAreaSizeX = (int)(m_fPageHeight * m_dbScaleFator);
		m_iRenderAreaSizeY = (int)(m_fPageWidth * m_dbScaleFator);
	}
	else
	{
		if (m_bFitWidth)
			m_dbScaleFator = m_dbCommonFitWidthScale;
		if (m_bFitHeight)
			m_dbScaleFator = m_dbCommonFitHeightScale;

		m_iRenderAreaSizeX = (int)(m_fPageWidth * m_dbScaleFator);
		m_iRenderAreaSizeY = (int)(m_fPageHeight* m_dbScaleFator);
	}
}

void renderPage::Click_BTN_RotateLeft(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to rotate the page counterclockwise in view.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
	m_iRotation = (m_iRotation + 3) % 4;
	ShowPage();
}

void renderPage::Click_BTN_RotateRight(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to rotate the page clockwise in view.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
	m_iRotation = (m_iRotation + 1) % 4;
	ShowPage();
}

void renderPage::Click_BTN_ZoomOut(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to zoom out the page.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}

	if (m_dbScaleFator - m_dbScaleDelta < 0.1)
		return;
	
	m_bFitHeight = false;
	m_bFitWidth = false;
	m_dbScaleFator -= m_dbScaleDelta;
	ShowPage();
}

void renderPage::Click_BTN_ZoomIn(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to zoom in the page.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}
	
	if (m_dbScaleFator + m_dbScaleDelta > 1.5)
		return;
	
	m_bFitHeight = false;
	m_bFitWidth = false;
	m_dbScaleFator += m_dbScaleDelta;
	ShowPage();
}

void renderPage::Click_BTN_ActualSize(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to show the page as its actual size.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}

	m_bFitHeight = false;
	m_bFitWidth = false;
	m_dbScaleFator = 1.0;
	ShowPage();
}

void renderPage::Click_BTN_FitWidth(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to fit the page show size to screen width.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}

	m_bFitHeight = false;
	m_bFitWidth = true;
	ShowPage();
}

void renderPage::Click_BTN_FitHeight(Platform::Object^ sender, Windows::UI::Xaml::RoutedEventArgs^ e)
{// Button click event: to fit the page show size to screen height.
	if (NULL == m_PDFDoc || NULL == m_PDFPage)
	{
		ShowErrorLog("Error: No PDF document or page has been loaded successfully.", ref new UICommandInvokedHandler(this, &demo_view::renderPage::ReturnCommandInvokedHandler));
		return;
	}

	m_bFitHeight = true;
	m_bFitWidth = false;
	ShowPage();
}

void renderPage::ReturnCommandInvokedHandler(Windows::UI::Popups::IUICommand^ command)
{
	if (true == m_bReleaseLibrary)
		ReleaseResources();
	if (Frame->CanGoBack)
	{
		Frame->GoBack();
	}
}


