﻿#include "pch.h"
#include "Crt.h"
#include "Util.h"

using namespace FSDKDLL::FSDK;

///////////////////////
//Class FSDKDLL::FSDK::RectI
Boolean	RectI::Contains(int32 x, int32 y)
{
	if (Top <= Bottom)
		return x >= Left && x < Right && y >= Top && y < Bottom;
	else
		return x >= Left && x < Right && y >= Bottom && y < Top;
}

void	RectI::Intersect(RectI^ rt)
{
	if (Left < rt->Left) Left = rt->Left;
	if (Right > rt->Right) Right = rt->Right;
	if (Top < Bottom)
	{
		if (Top < rt->Top) Top = rt->Top;
		if (Bottom > rt->Bottom) Bottom = rt->Bottom;
	}
	else
	{
		if (Top > rt->Top) Top = rt->Top;
		if (Bottom < rt->Bottom) Bottom = rt->Bottom;
	}
}

Boolean	RectI::IntersectsWith(RectI^ rt)
{
	rt->Intersect(this);
	return !rt->IsEmpty();
}

Boolean	RectI::IsEmpty()
{
	if ((Right - Left) <= 0 || 0 == abs(Top - Bottom))
		return true;
	else
		return false;
}

void	RectI::Empty()
{
	Left = 0;
	Top = 0;
	Right = 0;
	Bottom = 0;
}

void	RectI::Union(RectI^ rt)
{
	if (Left > rt->Left) Left = rt->Left;
	if (Right < rt->Right) Right = rt->Right;
	if (Top < Bottom)
	{
		if (Top > rt->Top) Top = rt->Top;
		if (Bottom < rt->Bottom) Bottom = rt->Bottom;
	}
	else
	{
		if (Top < rt->Top) Top = rt->Top;
		if (Bottom > rt->Bottom) Bottom = rt->Bottom;
	}
}

///////////////////////
//Class FSDKDLL::FSDK::RectF
Boolean	RectF::Contains(float32 x, float32 y)
{
	if (Top <= Bottom)
		return x >= Left && x < Right && y >= Top && y < Bottom;
	else
		return x >= Left && x < Right && y >= Bottom && y < Top;
}

void	RectF::Intersect(RectF^ rt)
{
	if (Left < rt->Left) Left = rt->Left;
	if (Right > rt->Right) Right = rt->Right;
	if (Top < Bottom)
	{
		if (Top < rt->Top) Top = rt->Top;
		if (Bottom > rt->Bottom) Bottom = rt->Bottom;
	}
	else
	{
		if (Top > rt->Top) Top = rt->Top;
		if (Bottom < rt->Bottom) Bottom = rt->Bottom;
	}
}

Boolean	RectF::IntersectsWith(RectF^ rt)
{
	rt->Intersect(this);
	return !rt->IsEmpty();
}

Boolean	RectF::IsEmpty()
{
	if ((Right - Left) <= 0 || 0 == fabs(Top - Bottom)< 0.000001)
		return true;
	else
		return false;
}

void	RectF::Empty()
{
	Left = 0.0f;
	Top = 0.0f;
	Right = 0.0f;
	Bottom = 0.0f;
}

void	RectF::Union(RectF^ rt)
{
	if (Left > rt->Left) Left = rt->Left;
	if (Right < rt->Right) Right = rt->Right;
	if (Top < Bottom)
	{
		if (Top > rt->Top) Top = rt->Top;
		if (Bottom < rt->Bottom) Bottom = rt->Bottom;
	}
	else
	{
		if (Top < rt->Top) Top = rt->Top;
		if (Bottom > rt->Bottom) Bottom = rt->Bottom;
	}
}

//////////////////////
//Class FSDKDLL::FSDK::File
File::File()
{
	m_pFileRW = NULL;
	m_pFileStream = NULL;
	m_hFile.pointer = 0;
}

Boolean File::Create(StorageFile^ pdfFile, int32 iFileSize)
{
	if (nullptr == pdfFile)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	Release();
	m_pFileRW = new CMy_FileReadWrite();
	m_pFileRW->CreateFileRead(pdfFile, iFileSize);
	m_pFileStream = new CMy_File();
	m_pFileStream->InitFileHandle(m_pFileRW);

	FSCRT_FILE fsFile = NULL;
	FS_RESULT ret = FSCRT_File_Create(m_pFileStream, &fsFile);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		m_hFile.pointer = (int64)fsFile;
		return true;
	}
	else
		return false;
}

Boolean File::Release()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	if (m_hFile.pointer)
	{
		ret = FSCRT_File_Release((FSCRT_FILE)m_hFile.pointer);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
			return false;
		}
	}
	m_hFile.pointer = 0;

	if (m_pFileStream)
		delete m_pFileStream;

	m_pFileStream = NULL;
	m_pFileRW = NULL;
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_SUCCESS, NULL);
	return true;
}

IObservableVector<Object^>^ File::GetFileBuffer()
{
	if (NULL == m_pFileRW)
		return nullptr;

	return m_pFileRW->GetFileBuffer();
}

/////////////////////
//Class FSDKDLL::FSDK::Crt

Boolean Crt::InitializeLibrary()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSCRT_Library_CreateDefaultMgr();
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean Crt::UnlockLibrary(String^ licenseKey, String^ unlockCode)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	auto sdkLicenseKey = ConvertToFoxitString(licenseKey);
	auto sdkUnlockcode = ConvertToFoxitString(unlockCode);
	ret = FSCRT_License_UnlockLibrary(&*sdkLicenseKey, &*sdkUnlockcode);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean Crt::LoadSystemFont()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSCRT_Library_LoadSystemFonts();
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

void Crt::ReleaseLibrary()
{
	FSCRT_Library_DestroyMgr();
}

LicenseType Crt::GetLicenseType()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	FS_INT32 iLicType = 0;
	ret = FSCRT_License_GetType(&iLicType);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return (LicenseType)iLicType;
}

ErrorStatus Crt::GetLastError()
{
	FS_LPVOID pData = NULL;
	FSCRT_Library_GetPrivateData(ERRORCODE_MODULE, &pData);
	return (ErrorStatus)(int)(pData);
}

FontHandle Crt::CreateFont(String^ fontName, FontStyle fontStyle, int32 weight, Charset charset)
{
	FontHandle font;
	font.pointer = 0;
	auto fontname = ConvertToFoxitString(fontName);
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	FSCRT_FONT pdffont = NULL;
	ret = FSCRT_Font_Create(&*fontname, (FS_DWORD)fontStyle, (FS_INT32)weight, (FS_INT32)charset, &pdffont);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		font.pointer = (int64)pdffont;
	return font;
}

Boolean Crt::ReleaseFont(FontHandle font)
{
	if (font.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSCRT_Font_Release((FSCRT_FONT)font.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}
