﻿#pragma once

#ifndef _CRT_H_
#define _CRT_H_

#include "pch.h"
#include <stdio.h>
#include <iostream>
#include <vector>
#include <ppltasks.h>
#include "Util.h"

using namespace Platform;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage;
using namespace Windows::Foundation;
using namespace Windows::Foundation::Collections;

//For internal use
#define		WINRT_MODULE_BASE			100
#define		ERRORCODE_MODULE			(FS_LPVOID)(WINRT_MODULE_BASE+10)


namespace FSDKDLL
{
	namespace FSDK
	{
		//////////////////////////// Handle ///////////////////////////
		/** @brief File handle. */
		public value struct FileHandle { int64 pointer; /**< @brief The value of the pointer to SDK file handle */ };

		/** @brief Document handle. */
		public value struct DocHandle { int64 pointer; /**< @brief The value of the pointer to SDK document handle*/ };

		/** @brief Page handle. */
		public value struct PageHandle { int64 pointer; /**< @brief The value of the pointer to SDK page handle*/ };

		/** @brief Font handle. */
		public value struct FontHandle { int64 pointer; /**< @brief The value of the pointer to SDK font handle*/ };

		/** @brief Annotation handle */
		public value struct AnnotHandle { int64 pointer; /**< @brief The value of the pointer to SDK annotation handle */ };

		/** @brief PDF action handle. */
		public value struct ActionHandle { int64 pointer; /**< @brief The value of the pointer to SDK action handle */ };

		/** @brief Text Page handle. */
		public value struct TextPageHandle { int64 pointer; /**< @brief The value of the pointer to SDK PDF text page handle */ };

		/** @brief Text Selection handle. */
		public value struct TextSelectionHandle { int64 pointer; /**< @brief The value of the pointer to SDK PDF text selection handle */ };

		public value struct TextSearchHandle { int64 pointer; /**< @brief The value of the pointer to SDK PDF text search handle */ };

		/** @brief Error Status Flags */
		public enum class ErrorStatus
		{
			Success = FSCRT_ERRCODE_SUCCESS,			/**< @brief Success. */
			Error = FSCRT_ERRCODE_ERROR,				/**< @brief	Common error for any kind. */
			InvalidManager = FSCRT_ERRCODE_INVALIDMANAGER,		/**< @brief	Invalid Foxit PDF SDK manager. */
			InvalidModule = FSCRT_ERRCODE_INVALIDMODULE,		/**< @brief	Invalid module or module uninitialized. */
			UnRecoverable = FSCRT_ERRCODE_UNRECOVERABLE,		/**< @brief	The function can't be recovered. */
			Memory = FSCRT_ERRCODE_OUTOFMEMORY,		/**< @brief	It runs out of memory. */
			Password = FSCRT_ERRCODE_PASSWORD,			/**< @brief Incorrect password. */
			Format = FSCRT_ERRCODE_FORMAT,				/**< @brief Data format error. */
			File = FSCRT_ERRCODE_FILE,				/**< @brief File access error, cannot read or write file data. */
			Param = FSCRT_ERRCODE_PARAM,				/**< @brief Parameter error. */
			InvalidLicense = FSCRT_ERRCODE_INVALIDLICENSE,		/**< @brief License authorization error. */
			Handler = FSCRT_ERRCODE_HANDLER,			/**< @brief	Callback functions in handler are error. */
			UnknownSecurityHandler = FSCRT_ERRCODE_UNKNOWNSECURITYHANDLER,		/**< @brief	Unknown security handler or cannot find availably one. */
			InvalidCertificate = FSCRT_ERRCODE_INVALIDCERTIFICATE,	/**< @brief	Incorrect certificate or its data error. */
			NotFound = FSCRT_ERRCODE_NOTFOUND,			/**< @brief Data cannot be found. */
			InvalidType = FSCRT_ERRCODE_INVALIDTYPE,		/**< @brief	Invalid object type. */
			Unsupport = FSCRT_ERRCODE_UNSUPPORTED,		/**< @brief	Some features are not implemented, cannot be used. */
			NotParsed = FSCRT_ERRCODE_NOTPARSED,			/**< @brief	Contents are not parsed. */
			UnknownStatus = FSCRT_ERRCODE_UNKNOWNSTATE,		/**< @brief	State is error or unknown. */
			BufferOverFlow = FSCRT_ERRCODE_BUFFEROVERFLOW,		/**< @brief	Buffer is overflow, need more space. */
			Conflict = FSCRT_ERRCODE_CONFLICT,			/**< @brief	Data or values conflict. */
			DataNotReady = FSCRT_ERRCODE_DATANOTREADY,		/**< @brief	Data not ready. */
			MemoryRebuilt = FSCRT_ERRCODE_MEMORYREBUILT,		/**< @brief	Memory is rebuilded. */
			TobeContinued = FSCRT_ERRCODE_TOBECONTINUED,		/**< @brief To be continued. */
			Finished = FSCRT_ERRCODE_FINISHED,			/**< @brief	Current process has finished, need not do again. */
			RollBack = FSCRT_ERRCODE_ROLLBACK			/**< @brief	To rollback the application process. */
		};

		/** @brief Font Style Flags */
		public enum class FontStyle
		{
			FixedPitch = FSCRT_FONTSTYLE_FIXEDPITCH,		/**< @brief Fixed pitch. */
			Serif = FSCRT_FONTSTYLE_SERIF,			/**< @brief Serif. */
			Symbolic = FSCRT_FONTSTYLE_SYMBOLIC,			/**< @brief Symbolic. */
			Script = FSCRT_FONTSTYLE_SCRIPT,			/**< @brief Script. */
			NonSymbolic = FSCRT_FONTSTYLE_NONSYMBOLIC,		/**< @brief Non-symbolic. */
			Italic = FSCRT_FONTSTYLE_ITALIC,			/**< @brief Italic. */
			AllCap = FSCRT_FONTSTYLE_ALLCAP,			/**< @brief All cap. */
			SmallCap = FSCRT_FONTSTYLE_SMALLCAP,			/**< @brief Small cap. */
			Bold = FSCRT_FONTSTYLE_BOLD				/**< @brief Force bold. */
		};

		/** @brief Charset Flags */
		public enum class Charset
		{
			Ansi = FSCRT_CHARSET_ANSI,			/**< @brief	ANSI (United States, Western Europe). */
			Default = FSCRT_CHARSET_DEFAULT,		/**< @brief System default, for unknown or mapping purpose. */
			Symbol = FSCRT_CHARSET_SYMBOL,			/**< @brief	Standard symbols. */
			Japanese = FSCRT_CHARSET_SHIFTJIS,		/**< @brief	Japanese (Shift-JIS). */
			Korea = FSCRT_CHARSET_HANGEUL,		/**< @brief	Korean (Wansung). */
			GB2312 = FSCRT_CHARSET_GB2312,			/**< @brief Simplified Chinese. */
			BIG5 = FSCRT_CHARSET_CHINESEBIG5,	/**< @brief Traditional Chinese. */
			Thai = FSCRT_CHARSET_THAI,			/**< @brief	Thai. */
			EastEurope = FSCRT_CHARSET_EASTEUROPE,     /**< @brief	Eastern European. */
			Russian = FSCRT_CHARSET_RUSSIAN,		/**< @brief	Russian. */
			Greek = FSCRT_CHARSET_GREEK,			/**< @brief	Greek. */
			Turkish = FSCRT_CHARSET_TURKISH,		/**< @brief	Turkish. */
			Hebrew = FSCRT_CHARSET_HEBREW,			/**< @brief	Hebrew. */
			Arabic = FSCRT_CHARSET_ARABIC,			/**< @brief	Arabic. */
			Baltic = FSCRT_CHARSET_BALTIC			/**< @brief	Baltic. */
		};

		/** @brief License Type Flags */
		public enum class LicenseType
		{
			/**
			* @brief	Invalid license.
			*
			* @details	This means applications don't call the function Base::UnlockLibrary or license key/code is incorrect.
			*/
			Invalid = FSCRT_LICENSETYPE_INVALID,
			/**
			* @brief	Authorized license.
			*
			* @details	This means applications can use Foxit PDF SDK normally.
			*/
			Authorized = FSCRT_LICENSETYPE_AUTHORIZED,
			/**
			* @brief	Evaluation license.
			*
			* @details	This means applications are using an evaluation version of Foxit PDF SDK.
			*/
			Evaluation = FSCRT_LICENSETYPE_EVALUATION,
			/**
			* @brief	Expired license.
			*
			* @details	If applications use Foxit PDF SDK in the expired condition, please contact Foxit Software Inc. to get a legal license.
			*/
			Expired = FSCRT_LICENSETYPE_EXPIRED
		};

		/**
		* @brief Structure for date and time.
		* @note This is a wrapper of FSCRT_DATETIMEZONE in Foxit PDF SDK(C API)
		*/
		public value struct DateTimeZone
		{
			uint16				year;			/**< @brief Year, a four-digit number, such as 2014. */
			uint16				month;			/**< @brief Month, from 1 to 12. */
			uint16				day;			/**< @brief Day of month, from 1 to 31. */
			uint16				dayOfWeek;		/**< @brief Day of week, from 0 to 6. PDF standard doesn't support now, you can omit this field if it is not applicable. */
			uint16				hour;			/**< @brief Hour, from 0 to 23. */
			uint16				minute;			/**< @brief Minute, from 0 to 59. */
			uint16				second;			/**< @brief Second, from 0 to 60. 60 for leap second. */
			uint16				milliseconds;	/**< @brief Millisecond, from 0 to 999. PDF standard doesn't support now, you can omit this field if it is not applicable. */
												/**
												* @brief	Hour of time zone, from -12 to 12.
												*
												* @details	Positive value for the eastern time zone, and negative value for the western time zone.
												*/
			int16				tzHour;
			uint16				tzMinute;		/**< @brief Minute of time zone, from 0 to 59. */
		};

		//////////////////////////////Class//////////////////////////////////

		/** @brief A point(integer) in device or page coordination system */
		public ref class PointI sealed
		{
		public:
			/** @brief Default constructor function */
			PointI() { X = 0; Y = 0; }
			/** @brief Constructor function with specified values. */
			PointI(int32 x, int32 y) { X = x; Y = y; }
			property int32	X;					/**< @brief The x-coordinate value. */
			property int32	Y;					/**< @brief The y-coordinate value. */
		};

		/** @brief A point(float) in device or page coordination system */
		public ref class PointF sealed
		{
		public:
			/** @brief Default constructor function */
			PointF() { X = 0.0f; Y = 0.0f; }
			/** @brief Constructor function with specified values. */
			PointF(float32 x, float32 y) { X = x; Y = y; }
			property float32	X;				/**< @brief The x-coordinate value. */
			property float32	Y;				/**< @brief The y-coordinate value. */
		};

		/**  @brief Class for Rectangle area(integer) in device or page coordination system. */
		public ref class RectI sealed
		{
		public:
			/** @brief Default constructor function */
			RectI()
			{
				Left = 0;
				Top = 0;
				Right = 0;
				Bottom = 0;
			}
			/** @brief Constructor function with specified values. */
			RectI(int32 left, int32 top, int32 right, int32 bottom)
			{
				Left = left;
				Top = top;
				Right = right;
				Bottom = bottom;
			}

			/**
			* @brief Indicate whether the rectangle contains the specified x-coordinate and y-coordinate.
			*
			* @param[in] x		The value of specified x-coordinate of the point to check.
			* @param[in] y		The value of specified y-coordinate of the point to check.
			*
			* @return <b>true</b> means (x,y) is contained by the rectangle; otherwise, <b>false</b>.
			*/
			Boolean		Contains(int32 x, int32 y);

			/**
			* @brief Find the intersection of the current rectangle and the specified rectangle, and stores the result as the current rectangle.
			*
			* @param[in] rt		The rectangle to intersect with the current rectangle.
			*
			* @return None.
			*/
			void		Intersect(RectI^ rt);

			/**
			* @brief Indicate whether the specified rectangle intersects with the current rectangle.
			*
			* @param[in] rt		The rectangle to check.
			*
			* @return <b>true</b> if the specified rectangle intersects with the current rectangle; otherwise, <b>false</b>.
			*/
			Boolean		IntersectsWith(RectI^ rt);

			/**
			* @brief Get a value that indicates whether the rectangle is the Empty rectangle.
			*
			* @return <b>true</b> if the rectangle is the Empty rectangle; otherwise, <b>false</b>.
			*/
			Boolean		IsEmpty();

			/**
			* @brief Make the rectangle empty.
			*
			* @return None.
			*/
			void		Empty();

			/**
			* @brief Expand the current rectangle exactly enough to contain the specified rectangle.
			*
			* @param[in] rt		The rectangle to include.
			*
			* @return None.
			*/
			void		Union(RectI^ rt);


			property int32	Left;						/**< @brief The x-coordinate of the left-top corner. */
			property int32	Top;						/**< @brief the y-coordinate of the left-top corner. */
			property int32	Right;						/**< @brief The x-coordinate of the right-bottom corner. */
			property int32	Bottom;						/**< @brief The y-coordinate of the right-bottom corner. */
		};

		/**  @brief Class for Rectangle area(float) in device or page coordination system */
		public ref class RectF sealed
		{
		public:
			/** @brief Default constructor function */
			RectF()
			{
				Left = 0.0f;
				Top = 0.0f;
				Right = 0.0f;
				Bottom = 0.0f;
			}
			/** @brief Constructor function with specified values. */
			RectF(float32 left, float32 top, float32 right, float32 bottom)
			{
				Left = left;
				Top = top;
				Right = right;
				Bottom = bottom;
			}

			/**
			* @brief Indicate whether the rectangle contains the specified x-coordinate and y-coordinate.
			*
			* @param[in] x		The value of specified x-coordinate of the point to check.
			* @param[in] y		The value of specified y-coordinate of the point to check.
			*
			* @return <b>true</b> means (x,y) is contained by the rectangle; otherwise, <b>false</b>.
			*/
			Boolean		Contains(float32 x, float32 y);

			/**
			* @brief Find the intersection of the current rectangle and the specified rectangle, and stores the result as the current rectangle.
			*
			* @param[in] rt		The rectangle to intersect with the current rectangle.
			*
			* @return None.
			*/
			void		Intersect(RectF^ rt);

			/**
			* @brief Indicate whether the specified rectangle intersects with the current rectangle.
			*
			* @param[in] rt		The rectangle to check.
			*
			* @return <b>true</b> if the specified rectangle intersects with the current rectangle; otherwise, <b>false</b>.
			*/
			Boolean		IntersectsWith(RectF^ rt);

			/**
			* @brief Get a value that indicates whether the rectangle is the Empty rectangle.
			*
			* @return <b>true</b> if the rectangle is the Empty rectangle; otherwise, <b>false</b>.
			*/
			Boolean		IsEmpty();

			/**
			* @brief Make the rectangle empty.
			*
			* @return None.
			*/
			void		Empty();

			/**
			* @brief Expand the current rectangle exactly enough to contain the specified rectangle.
			*
			* @param[in] rt		The rectangle to include.
			*
			* @return None.
			*/
			void		Union(RectF^ rt);

			property float32	Left;				/**< @brief The x-coordinate of the left-top corner. */
			property float32	Top;				/**< @brief The y-coordinate of the left-top corner. */
			property float32	Right;				/**< @brief The x-coordinate of the right-bottom corner. */
			property float32	Bottom;				/**< @brief The y-coordinate of the right-bottom corner. */
		};

		/** @brief Class for matrix which is used for transformation. */
		public ref class Matrix sealed
		{
		public:
			/** @brief Default constructor function */
			Matrix()
			{
				A = 1.0f;
				B = 0.0f;
				C = 0.0f;
				D = 1.0f;
				E = 0.0f;
				F = 0.0f;
			}
			/** @brief Constructor function with specified values. */
			Matrix(float32 a, float32 b, float32 c, float32 d, float32 e, float32 f)
			{
				A = a;
				B = b;
				C = c;
				D = d;
				E = e;
				F = f;
			}

			property float32	A;					/**< @brief Coefficient a.*/
			property float32	B;					/**< @brief Coefficient b.*/
			property float32	C;					/**< @brief Coefficient c.*/
			property float32	D;					/**< @brief Coefficient d.*/
			property float32	E;					/**< @brief Coefficient e.*/
			property float32	F;					/**< @brief Coefficient f.*/
		};

		/**
		* @brief Class for file handle control.
		* @note This class is a wrapper of FSCRT_FILE and its relative methods in Foxit PDF SDK(C API).
		*/
		public ref class File sealed
		{
		public:
			File();

			/**
			* @brief	Create a file handle.
			*
			* @details	The created file handle can be got by calling function DemoRTSDK::Foxit::File::GetFileHandle.
			*			And when no need to use the file handle, please call function DemoRTSDK::Foxit::File::Release to release relative resources.
			*
			* @param[in]	pdfFile		The file item.
			* @param[in]	iFileSize	The file size got from <i>pdfFile</i>.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*/
			Boolean Create(StorageFile^ pdfFile, int32 iFileSize);

			/**
			* @brief	Release the file handle.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*/
			Boolean Release();

			/**
			* @brief	Get the file handle.
			*
			* @return	The file handle.
			*/
			FileHandle	GetFileHandle() { return m_hFile; }

			/**
			* @brief	Get file buffer.
			*
			* @return	The file buffer, which is in byte.
			*/
			IObservableVector<Object^>^ GetFileBuffer();

		private:
			property CMy_FileReadWrite*	m_pFileRW;
			property CMy_File*			m_pFileStream;
			FileHandle					m_hFile;
		};

		/**
		* @brief Class for Base module.
		* @note This class is a wrapper for some methods defined in fs_base_r.h of Foxit PDF SDK(C API).
		*/
		public ref class Crt sealed
		{
		public:
			/**
			* @brief	Initialize Foxit PDF SDK library.
			*
			* @details	This should be called before Foxit PDF SDK(C API) library can be used.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_Library_CreateDefaultMgr in Foxit PDF SDK(C API).
			*/
			static Boolean InitializeLibrary();

			/**
			* @brief	Unlock Foxit PDF SDK library.
			*
			* @param[in]	licenseKey	License key string.
			* @param[in]	unlockCode	Unlock code string.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_License_UnlockLibrary in Foxit PDF SDK(C API).
			*/
			static Boolean UnlockLibrary(String^ licenseKey, String^ unlockCode);

			/**
			* @brief	Get the license type.
			*
			* @details	This function is used after calling FSDKDLL::FSDK::Crt::UnlockLibrary.
			*
			* @return	A LicenseType object.
			*
			* @note	This function is a wrapper of FSCRT_License_GetType in Foxit PDF SDK(C API).
			*/
			static LicenseType GetLicenseType();

			/**
			* @brief	Load system font in Windows Phone Platform.
			*
			* @details	Usually, this is necessary to load system font for Foxit PDF SDK(C API) using them to analyze contents of a PDF file.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_Font_LoadSystemFont in Foxit PDF SDK(C API).
			*/
			static Boolean LoadSystemFont();

			/**
			* @brief	Release Foxit PDF SDK library.
			*
			* @note	This function is a wrapper of FSCRT_Library_DestroyMgr in Foxit PDF SDK(C API).
			*/
			static void ReleaseLibrary();

			/**
			* @brief	Get last error status.
			*
			* @details	This can be used after failing to call most methods in this component, in order to know more about the reason why it fails.
			*
			* @return	A ErrorStatus object.
			*/
			static ErrorStatus GetLastError();

			/**
			* @brief	Create a font with given attributes.
			*
			* @param[in]	fontName	Font name, which specifies a typeface name of a given font.
			* @param[in]	fontStyle	Font style. Please refer to enum DemoRTSDK::Foxit::FontStyle.
			* @param[in]	weight		Original font weight. 0 means unspecified.
			* @param[in]	charset		The charset of the font. Please refer to enum DemoRTSDK::Foxit::Charset.
			*
			* @return	A font handle.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_Font_Create in Foxit PDF SDK(C API).
			*/
			static FontHandle CreateFont(String^ fontName, FontStyle fontStyle, int32 weight, Charset charset);

			/**
			* @brief	Release a font handle.
			*
			* @param[in]	font	A font handle.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_Font_Release in Foxit PDF SDK(C API).
			*/
			static Boolean ReleaseFont(FontHandle font);

		};
	}
}

#endif //_CRT_H_