#include "pch.h"
#include "PDFAnnot.h"
#include "Util.h"

using namespace Platform;
using namespace FSDKDLL::FSDK;

///////////////////////////////////////
//Class FSDKDLL::FSDK::PDFAnnot

Boolean PDFAnnot::Remove()
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_Remove((FSCRT_ANNOT)m_pdfAnnot.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::ResetAppearance()
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_ResetAppearance((FSCRT_ANNOT)m_pdfAnnot.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::Move(RectF^ rectf)
{
	if (0 == m_pdfAnnot.pointer || nullptr == rectf)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSCRT_RECTF fsRectF;
	fsRectF.bottom = rectf->Bottom;
	fsRectF.left = rectf->Left;
	fsRectF.right = rectf->Right;
	fsRectF.top = rectf->Top;

	FS_RESULT ret = FSPDF_Annot_Move((FSCRT_ANNOT)m_pdfAnnot.pointer, &fsRectF);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::InsertAction(AnnotTrigger trigger, int32 index, ActionHandle action)
{
	if (0 == m_pdfAnnot.pointer || 0 == action.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FS_RESULT ret = FSPDF_Annot_InsertAction((FSCRT_ANNOT)m_pdfAnnot.pointer, (FS_INT32)trigger, (FS_INT32)index, (FSPDF_ACTIONDATA *)action.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::RemoveAction(AnnotTrigger trigger, int32 index)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_RemoveAction((FSCRT_ANNOT)m_pdfAnnot.pointer, (FS_INT32)trigger, (FS_INT32)index);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetFlags(int32 flags)//AnnotFlag
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetFlags((FSCRT_ANNOT)m_pdfAnnot.pointer, flags);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetName(String^ name)
{
	if (0 == m_pdfAnnot.pointer || nullptr == name)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	auto bsName = ConvertToFoxitString(name);
	FS_RESULT ret = FSPDF_Annot_SetName((FSCRT_ANNOT)m_pdfAnnot.pointer, &*bsName);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetSubject(String^ subject)
{
	if (0 == m_pdfAnnot.pointer || nullptr == subject)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	auto bsSubject = ConvertToFoxitString(subject);
	FS_RESULT ret = FSPDF_Annot_SetSubject((FSCRT_ANNOT)m_pdfAnnot.pointer, &*bsSubject);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetTitle(String^ title)
{
	if (0 == m_pdfAnnot.pointer || nullptr == title)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	auto bsTitle = ConvertToFoxitString(title);
	FS_RESULT ret = FSPDF_Annot_SetTitle((FSCRT_ANNOT)m_pdfAnnot.pointer, &*bsTitle);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetContents(String^ content)
{
	if (0 == m_pdfAnnot.pointer || nullptr == content)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	auto bsContent = ConvertToFoxitString(content);
	FS_RESULT ret = FSPDF_Annot_SetContents((FSCRT_ANNOT)m_pdfAnnot.pointer, &*bsContent);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetBorder(AnnotBorder^ border)
{
	if (0 == m_pdfAnnot.pointer || nullptr == border)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSPDF_ANNOTBORDER annotBorder;
	annotBorder.borderStyle = (int32)border->borderStyle;
	annotBorder.borderWidth = border->borderWidth;
	annotBorder.cloudIntensity = border->cloudIntensity;
	annotBorder.dashPhase = border->dashPhase;
	annotBorder.dashCount = border->dashCount;
	//The length of array dashPattern is 16. So the dashcount can't bigger than 16
	if (border->dashCount <= 16)
		annotBorder.dashCount = border->dashCount;
	else
		annotBorder.dashCount = 16;
	for (int i = 0; i < annotBorder.dashCount; i++)
	{
		annotBorder.dashPattern[i] = border->dashPattern[i];
	}
	FS_RESULT ret = FSPDF_Annot_SetBorder((FSCRT_ANNOT)m_pdfAnnot.pointer, &annotBorder);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetColor(Boolean isFillColor, uint32 color)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetColor((FSCRT_ANNOT)m_pdfAnnot.pointer, isFillColor, color);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetOpacity(float32 opacity)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetOpacity((FSCRT_ANNOT)m_pdfAnnot.pointer, opacity);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetAlignment(int32 alignment)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetAlignment((FSCRT_ANNOT)m_pdfAnnot.pointer, alignment);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetCreationDateTime(DateTimeZone time)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSCRT_DATETIMEZONE fsDatetime;
	fsDatetime.year = time.year;
	fsDatetime.month = time.month;
	fsDatetime.day = time.day;
	fsDatetime.dayOfWeek = time.dayOfWeek;
	fsDatetime.hour = time.hour;
	fsDatetime.minute = time.minute;
	fsDatetime.second = time.second;
	fsDatetime.milliseconds = time.milliseconds;
	fsDatetime.tzHour = time.tzHour;
	fsDatetime.tzMinute = time.tzMinute;
	FS_RESULT ret = FSPDF_Annot_SetCreationDateTime((FSCRT_ANNOT)m_pdfAnnot.pointer, &fsDatetime);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetModifiedDateTime(DateTimeZone time)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSCRT_DATETIMEZONE fsDatetime;
	fsDatetime.year = time.year;
	fsDatetime.month = time.month;
	fsDatetime.day = time.day;
	fsDatetime.dayOfWeek = time.dayOfWeek;
	fsDatetime.hour = time.hour;
	fsDatetime.minute = time.minute;
	fsDatetime.second = time.second;
	fsDatetime.milliseconds = time.milliseconds;
	fsDatetime.tzHour = time.tzHour;
	fsDatetime.tzMinute = time.tzMinute;
	FS_RESULT ret = FSPDF_Annot_SetModifiedDateTime((FSCRT_ANNOT)m_pdfAnnot.pointer, &fsDatetime);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetHighlightingMode(AnnotHighLightMode mode)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetHighlightingMode((FSCRT_ANNOT)m_pdfAnnot.pointer, (int32)mode);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetQuadPoints(const Array<AnnotQuad^>^ quadPoints, uint32 count)
{
	if (0 == m_pdfAnnot.pointer || nullptr == quadPoints || count <1)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSCRT_QUADPOINTSF *fsQuadPoints = new FSCRT_QUADPOINTSF[count];
	for (unsigned int i = 0; i < count; i++)
	{
		fsQuadPoints[i].x1 = quadPoints[i]->X1;
		fsQuadPoints[i].y1 = quadPoints[i]->Y1;
		fsQuadPoints[i].x2 = quadPoints[i]->X2;
		fsQuadPoints[i].y2 = quadPoints[i]->Y2;
		fsQuadPoints[i].x3 = quadPoints[i]->X3;
		fsQuadPoints[i].y3 = quadPoints[i]->Y3;
		fsQuadPoints[i].x4 = quadPoints[i]->X4;
		fsQuadPoints[i].y4 = quadPoints[i]->Y4;
	}

	FS_RESULT ret = FSPDF_Annot_SetQuadPoints((FSCRT_ANNOT)m_pdfAnnot.pointer, fsQuadPoints, (FS_INT32)count);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetLineEndingStyles(LineEndingStyle startingStyle, LineEndingStyle endingStyle)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	String^ startingstyle = LineEndingStyleToString(startingStyle);
	if (nullptr == startingstyle)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	String^ endingstyle = LineEndingStyleToString(endingStyle);
	if (nullptr == endingstyle)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	auto starting = ConvertToFoxitString(startingstyle);
	auto ending = ConvertToFoxitString(endingstyle);
	FS_RESULT ret = FSPDF_Annot_SetLineEndingStyles((FSCRT_ANNOT)m_pdfAnnot.pointer, &*starting, &*ending);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetLinePosition(PointF^ pointF1, PointF^ pointF2)
{
	if (0 == m_pdfAnnot.pointer || nullptr == pointF1 || nullptr == pointF2)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetLinePosition((FSCRT_ANNOT)m_pdfAnnot.pointer, pointF1->X, pointF1->Y, pointF2->X, pointF2->Y);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetLeaderLineLength(float32 length)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetLeaderLineLength((FSCRT_ANNOT)m_pdfAnnot.pointer, length);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetLeaderLineExtension(float32 extLength)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetLeaderLineExtension((FSCRT_ANNOT)m_pdfAnnot.pointer, extLength);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetLeaderLineOffset(float32 offset)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetLeaderLineOffset((FSCRT_ANNOT)m_pdfAnnot.pointer, offset);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetCaptionContents(Boolean hasCap)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetCaptionContents((FSCRT_ANNOT)m_pdfAnnot.pointer, hasCap);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetCaptionPosition(String^ captionPos)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	auto bsCaptionPos = ConvertToFoxitString(captionPos);
	FS_RESULT ret = FSPDF_Annot_SetCaptionPosition((FSCRT_ANNOT)m_pdfAnnot.pointer, &*bsCaptionPos);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetCaptionOffset(float32 offsetx, float32 offsety)
{
	if (0 == m_pdfAnnot.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Annot_SetCaptionOffset((FSCRT_ANNOT)m_pdfAnnot.pointer, offsetx, offsety);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetVertices(const Array<PointF^>^ vertices, uint32 count)
{
	if (0 == m_pdfAnnot.pointer || nullptr == vertices || count < 2)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_FLOAT *fsVertices = new FS_FLOAT[count * 2];
	for (int i = 0; i < (int)count; i++)
	{
		int iVerticesNum = i * 2;
		fsVertices[iVerticesNum] = (FS_FLOAT)vertices[i]->X;
		fsVertices[iVerticesNum + 1] = (FS_FLOAT)vertices[i]->Y;
	}
	FS_RESULT ret = FSPDF_Annot_SetVertices((FSCRT_ANNOT)m_pdfAnnot.pointer, fsVertices, (FS_INT32)(count * 2));
	delete vertices;
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFAnnot::SetDefAppearance(DefaultAppearance^ appearance)
{
	if (0 == m_pdfAnnot.pointer || nullptr == appearance || nullptr == appearance->textMatrix)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FSPDF_DEFAULTAPPEARANCE pDefAppearance;
	pDefAppearance.flags = appearance->flags;
	pDefAppearance.reserved = appearance->reserved;
	pDefAppearance.font = (FSCRT_FONT)appearance->font.pointer;
	pDefAppearance.fontSize = appearance->fontSize;
	pDefAppearance.textColor = appearance->textColor;

	FSCRT_MATRIX pMatrix;
	pMatrix.a = appearance->textMatrix->A;
	pMatrix.b = appearance->textMatrix->B;
	pMatrix.c = appearance->textMatrix->C;
	pMatrix.d = appearance->textMatrix->D;
	pMatrix.e = appearance->textMatrix->E;
	pMatrix.f = appearance->textMatrix->F;
	pDefAppearance.textMatrix = pMatrix;

	FS_RESULT ret = FSPDF_Annot_SetDefaultAppearance((FSCRT_ANNOT)m_pdfAnnot.pointer, &pDefAppearance);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

String^ PDFAnnot::LineEndingStyleToString(LineEndingStyle style)
{
	String^ typestr = ref new String();
	switch (style)
	{
		case LineEndingStyle::None:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_NONE;
			break;
		}
		case LineEndingStyle::Square:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_SQUARE;
			break;
		}
		case LineEndingStyle::Circle:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_CIRCLE;
			break;
		}
		case LineEndingStyle::Diamond:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_DIAMOND;
			break;
		}
		case LineEndingStyle::OpenArrow:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_OPENARROW;
			break;
		}
		case LineEndingStyle::ClosedArrow:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_CLOSEDARROW;
			break;
		}
		case LineEndingStyle::Butt:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_BUTT;
			break;
		}
		case LineEndingStyle::ROpenArrow:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_REVERSEOPENARROW;
			break;
		}
		case LineEndingStyle::RClosedArrow:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_REVERSECLOSEDARROW;
			break;
		}
		case LineEndingStyle::Slash:
		{
			typestr = FSPDF_ANNOT_LINEENDINGSTYLE_SLASH;
			break;
		}
		default:
		{
			return nullptr;
		}
	}

	return typestr;
}

