#pragma once

#ifndef _PDFANNOT_H_
#define _PDFANNOT_H_

#include "PDFBase.h"

using namespace Platform;

namespace FSDKDLL
{
	namespace FSDK
	{
		/** @brief Annotation Flags */
		public enum class AnnotFlag
		{
			Invisible = FSPDF_ANNOTFLAG_INVISIBLE,			/**< @brief Annotation flag: Invisible. */
			Hidden = FSPDF_ANNOTFLAG_HIDDEN,				/**< @brief Annotation flag: Hidden. */
			Print = FSPDF_ANNOTFLAG_PRINT,					/**< @brief Annotation flag: Print. */
			NoZoom = FSPDF_ANNOTFLAG_NOZOOM,				/**< @brief Annotation flag: NoZoom. */
			NoRotate = FSPDF_ANNOTFLAG_NOROTATE,			/**< @brief Annotation flag: NoRotate. */
			NoView = FSPDF_ANNOTFLAG_NOVIEW,				/**< @brief Annotation flag: NoView. */
			ReadOnly = FSPDF_ANNOTFLAG_READONLY,			/**< @brief Annotation flag: ReadOnly. */
			Locked = FSPDF_ANNOTFLAG_LOCKED,				/**< @brief Annotation flag: Locked. */
			ToggleNoView = FSPDF_ANNOTFLAG_TOGGLENOVIEW,	/**< @brief Annotation flag: ToggleNoView. */
			LocedContents = FSPDF_ANNOTFLAG_LOCKEDCONTENTS	/**< @brief Annotation flag: LockedContents. */
		};

		/** @brief Annotation Highlight Mode Flags. */
		public enum class AnnotHighLightMode
		{
			None = FSPDF_ANNOT_HIGHLIGHTINGMODE_NONE,			/**< @brief Highlighting mode: None */
			Invert = FSPDF_ANNOT_HIGHLIGHTINGMODE_INVERT,		/**< @brief Highlighting mode: Invert */
			OutLine = FSPDF_ANNOT_HIGHLIGHTINGMODE_OUTLINE,		/**< @brief Highlighting mode: Outline */
			Push = FSPDF_ANNOT_HIGHLIGHTINGMODE_PUSH,			/**< @brief Highlighting mode: Push */
			Toggle = FSPDF_ANNOT_HIGHLIGHTINGMODE_TOGGLE		/**< @brief Highlighting mode: Toggle */
		};

		/** @brief Trigger of Annotation Flags*/
		public enum class AnnotTrigger
		{
			Ce = FSPDF_TRIGGER_ANNOT_CE,	/**< brief An action to be performed when the cursor enters the annotation's active area. */
			Cx = FSPDF_TRIGGER_ANNOT_CX,	/**< brief An action to be performed when the cursor exits the annotation's active area. */
			Md = FSPDF_TRIGGER_ANNOT_MD,	/**< brief An action to be performed when the mouse button is pressed inside the annotation's active area. */
			Mu = FSPDF_TRIGGER_ANNOT_MU,	/**< brief An action to be performed when the mouse button is released inside the annotation's active area. */
			Fo = FSPDF_TRIGGER_ANNOT_FO,	/**< brief An action to be performed when the annotation receives the input focus. */
			Fl = FSPDF_TRIGGER_ANNOT_FL,	/**< brief An action to be performed when the annotation loses the input focus. */
			Po = FSPDF_TRIGGER_ANNOT_PO,	/**< brief An action to be performed when the page containing the annotation is opened. */
			Pc = FSPDF_TRIGGER_ANNOT_PC,	/**< brief An action to be performed when the page containing the annotation is closed. */
			Pv = FSPDF_TRIGGER_ANNOT_PV,	/**< brief An action to be performed when the page containing the annotation becomes visible in the viewer application's user interface. */
			Pi = FSPDF_TRIGGER_ANNOT_PI		/**< brief An action to be performed when the page containing the annotation is no longer visible in the viewer application's user interface. */
		};

		/** @brief Border Style Flags*/
		public enum class BorderStyle
		{
			Solid = FSPDF_ANNOT_BORDERSTYLE_SOLID,			/**< @brief	Border style: Solid */
			Dashed = FSPDF_ANNOT_BORDERSTYLE_DASHED,		/**< @brief	Border style: Dashed */
			Underline = FSPDF_ANNOT_BORDERSTYLE_UNDERLINE,	/**< @brief	Border style: Underline */
			Beveled = FSPDF_ANNOT_BORDERSTYLE_BEVELED,		/**< @brief	Border style: Beveled */
			Inset = FSPDF_ANNOT_BORDERSTYLE_INSET,			/**< @brief	Border style: Inset */
			Cloudy = FSPDF_ANNOT_BORDERSTYLE_CLOUDY			/**< @brief	Border style: Cloudy */
		};

		/** @brief Line Ending Style Flags*/
		public enum class LineEndingStyle
		{
			None = 0,			/**< @brief	Line annotation ending style name: None. */
			Square = 1,			/**< @brief	Line annotation ending style name: Square. */
			Circle = 2,			/**< @brief	Line annotation ending style name: Circle. */
			Diamond = 3,		/**< @brief	Line annotation ending style name: Diamond. */
			OpenArrow = 4,		/**< @brief	Line annotation ending style name: OpenArrow. */
			ClosedArrow = 5,	/**< @brief	Line annotation ending style name: ClosedArrow. */
			Butt = 6,			/**< @brief	Line annotation ending style name: Butt. */
			ROpenArrow = 7,		/**< @brief	Line annotation ending style name: ROpenArrow. */
			RClosedArrow = 8,	/**< @brief	Line annotation ending style name: RClosedArrow. */
			Slash = 9			/**< @brief	Line annotation ending style name: Slash. */
		};

		/**
		* @brief Class for Annotation Border
		* @note This is a wrapper of FSPDF_ANNOTBORDER in Foxit PDF SDK(C API)
		*/
		public ref class AnnotBorder sealed
		{
		public:
			AnnotBorder()
			{
				borderWidth = 0.0;
				borderStyle = BorderStyle::Solid;
				cloudIntensity = 0.0;
				dashPhase = 0.0;
				dashCount = 0;
			}
			/**
			* @brief	Width of border, in points.
			*
			* @details	If this value is 0, no border is drawn. This value should not be negative.
			*/
			property float32			borderWidth;
			/**
			* @brief	Border style.
			*
			* @details	This should be one of enum BorderStyle.
			*/
			property BorderStyle		borderStyle;
			/**
			* @brief	Intensity of the cloudy effect.
			*
			* @details	Only free text, square, circle, polygon, and polyline annotations can have cloud intensity property.<br>
			*			Suggested values range from 0 to 2. -1 mean no cloudy border effect.<br>
			*			Only useful when <i>borderStyle</i> is BorderStyle::Cloudy; otherwise -1.
			*/
			property float32			cloudIntensity;
			/**
			* @brief	Dash phase.
			*
			* @details	Only useful when <i>borderStyle</i> is BorderStyle::Dashed.
			*/
			property float32			dashPhase;
			/**
			* @brief	Count of useful elements in array <i>dashPattern</i>.
			*
			* @details	Only useful when <i>borderStyle</i> is BorderStyle::Dashed. This can't bigger than 16.
			*/
			property int32			dashCount;
			/**
			* @brief	Dash array.
			*
			* @details	The value of elements in this array should not be negative.<br>
			*			Only be useful when <i>borderStyle</i> is BorderStyle::Dashed.
			*/
			property Array<float32>^	dashPattern;
		};

		/**
		* @brief Class for quad points(in float) of an annotation.
		* @note This is a wrapper of FSCRT_QUADPOINTSF in Foxit PDF SDK(C API)
		*/
		public ref class AnnotQuad sealed
		{
		public:
			AnnotQuad() {}
			/** @brief	X coordinate for the first point. */
			property float32	X1;
			/** @brief	Y coordinate for the first point. */
			property float32	Y1;

			/** @brief	X coordinate for the second point. */
			property float32	X2;
			/** @brief	Y coordinate for the second point. */
			property float32	Y2;

			/** @brief	X coordinate for the third point. */
			property float32	X3;
			/** @brief	Y coordinate for the third point. */
			property float32	Y3;

			/** @brief	X coordinate for the fourth point. */
			property float32	X4;
			/** @brief	Y coordinate for the fourth point. */
			property float32	Y4;
		};


		/**
		* @brief Class for PDF Annotation module.
		* @note This class is a wrapper for some methods defined in fpdf_annot_r.h and fpdf_annot_w.h of Foxit PDF SDK(C API).
		*/
		public ref class PDFAnnot sealed
		{
		public:
			PDFAnnot(AnnotHandle pdfAnnot)
			{
				m_pdfAnnot = pdfAnnot;
			}

			AnnotHandle GetAnnotHandle()
			{
				return m_pdfAnnot;
			}
			/**
			* @brief	Remove an annotation from page.
			*
			* @param[in] annot		An annot handle.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_Remove in Foxit PDF SDK(C API).
			*/
			Boolean Remove();

			/**
			* @brief	Reset(regenerate) appearance of an annotation.
			*
			* @param[in] annot		An annot handle.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_ResetAppearance in Foxit PDF SDK(C API).
			*/
			Boolean ResetAppearance();

			/**
			* @brief	Move an annotation to a new position specified by a rectangle.
			*
			* @param[in] page		A page handle.
			* @param[in] rect		Reference a ref class RectF to specify the new position.
			*						The rect should be "normalized" for PDF page coordination: left is always smaller than right,
			*						and bottom is always smaller than top.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_Move in Foxit PDF SDK(C API).
			*/
			Boolean Move(RectF^ rectf);

			/**
			* @brief	Insert an action of specified index associated with an annotation.
			*
			* @param[in] annot		An annot handle.
			* @param[in] trigger	Specify the trigger defined in enum DemoRTSDK::Foxit::AnnotTrigger.
			* @param[in] index		Index of actions. Range: 0 to (actioncount-1). <i>actioncount</i> is returned by function :Annot::GetCount.
			* @param[in] action		An action handle created by function FSDKDLL::FSDK::Crt::GetActionData
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_InsertAction in Foxit PDF SDK(C API).
			*/
			Boolean InsertAction(AnnotTrigger trigger, int32 index, ActionHandle action);

			/**
			* @brief	Remove an action of specified index associated with an annotation.
			*
			* @param[in] annot		A page handle.
			* @param[in] trigger	Specify the trigger defined in enum Foxit::General::AnnotTrigger.
			* @param[in] index		Index of actions. Range: 0 to (actioncount-1). <i>actioncount</i> is returned by function ::Annot::GetCount.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_Annot_RemoveAction in Foxit PDF SDK(C API).
			*/
			Boolean RemoveAction(AnnotTrigger trigger, int32 index);

			///////////////////Setting Methods START////////////////////
			//ATTETION for following methods:
			//1. Following methods are all used for setting some property of an annotation.
			//2. They are wrappers of the C API in Foxit PDF SDK. Names of these C API include the function name in this component.
			//	 Here, only add comment for some special part in component definition of these function. 
			//	 Please refer to fpdf_annot_w.h to find the original C APIs and know more from their comment.
			//3. All of following methods use a boolean value as a return value: true means success, while false means failure.
			//	 When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			//4. For each of following method, only add some necessary

			// param[in] flags	It can be a combination of several value from DemoRTSDK::Foxit::AnnotFlag.
			Boolean SetFlags(int32 flags);

			Boolean SetName(String^ name);
			Boolean SetSubject(String^ subject);
			Boolean SetTitle(String^ title);
			Boolean SetContents(String^ content);
			Boolean SetBorder(AnnotBorder^ border);
			Boolean SetColor(Boolean isFillColor, uint32 color);
			Boolean SetOpacity(float32 opacity);

			// param[in] alignment	Alignment value to be set to the annotation. It should be one of the followings: <br>
			//						<ul>
			//						<li>0: Left-justified</li>
			//						<li>1: Centered</li>
			//						<li>2: Right-justified</li>
			//						</ul>
			Boolean SetAlignment(int32 alignment);

			Boolean SetCreationDateTime(DateTimeZone time);
			Boolean SetModifiedDateTime(DateTimeZone time);
			Boolean SetHighlightingMode(AnnotHighLightMode mode);

			// param[in] quadPoints		An array of quad points.
			// param[in] count			Count of the elements in array <i>quadPoints</i>.
			Boolean SetQuadPoints(const Array<AnnotQuad^>^ quadPoints, uint32 count);

			Boolean SetLineEndingStyles(LineEndingStyle startingStyle, LineEndingStyle endingStyle);
			Boolean SetLinePosition(PointF^ pointF1, PointF^ pointF2);
			Boolean SetLeaderLineLength(float32 length);
			Boolean SetLeaderLineExtension(float32 extLength);
			Boolean SetLeaderLineOffset(float32 offset);
			Boolean SetCaptionContents(Boolean hasCap);
			Boolean SetCaptionPosition(String^ captionPos);
			Boolean SetCaptionOffset(float32 offsetx, float32 offsety);

			// param[in] quadPoints		An array of PointF objects.
			// param[in] count			Count of the elements in array <i>vertices</i>.
			Boolean SetVertices(const Array<PointF^>^ vertices, uint32 count);

			Boolean SetDefAppearance(DefaultAppearance^ appearance);
			///////////////////Setting Methods END////////////////////

			/**
			* @brief	Convert value of LineEndingStyle to platform string.
			*
			* @param[in] style		Specify the stype of LineEnding defined in enum DemoRTSDK::Foxit::LineEndingStyle.
			*
			* @return	A string.
			*/
			String^ LineEndingStyleToString(LineEndingStyle style);

			private:
				property AnnotHandle m_pdfAnnot;
		};
	}
}
#endif //_PDFANNOT_H_