#include"pch.h"
#include "PDFBase.h"
#include "Util.h"

using namespace Platform;
using namespace FSDKDLL::FSDK;

///////////////////////////////////////
//Class DemoRTSDK::Foxit::PDFBase
Boolean PDFBase::InitializeModule()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSCRT_PDFModule_Initialize();
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

Boolean PDFBase::ReleaseModule()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSCRT_PDFModule_Finalize();
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

ActionHandle PDFBase::CreateActionData(ActionType type, Object^ data)
{
	ActionHandle action;
	action.pointer = 0;
	FSPDF_ACTIONDATA *actiondata = NULL;
	FS_RESULT ret = FSCRT_Memory_Alloc(sizeof(FSPDF_ACTIONDATA), (FS_LPVOID *)&actiondata);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		return action;
	}
	switch (type)
	{
	case ActionType::URI:
	{
		FSPDF_ACTIONDATA_URI *fsUrlData = NULL;
		ret = FSCRT_Memory_Alloc(sizeof(FSPDF_ACTIONDATA_URI), (FS_LPVOID *)&fsUrlData);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
			return action;
		}

		ActionDataURI^ urldata = (ActionDataURI ^)data;
		auto urlstr = ConvertToFoxitString(urldata->url);
		FSCRT_BStr_Init(&fsUrlData->url);
		FSCRT_BStr_Set(&fsUrlData->url, (*urlstr).str, (*urlstr).len);
		fsUrlData->mapPosition = urldata->mapPosition;

		actiondata->actionData = fsUrlData;
		actiondata->actionType = (FS_INT32)type;
		action.pointer = (int64)actiondata;
		break;
	}
	default:
		FSCRT_Memory_Free((FS_LPVOID)actiondata);
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_UNSUPPORTED, NULL);
		break;
	}
	return action;
}

void PDFBase::ReleaseActionData(ActionHandle action)
{
	if (action.pointer == 0)
		return;
	FSPDF_ACTIONDATA *actiondata = (FSPDF_ACTIONDATA *)action.pointer;
	if (actiondata->actionData)
	{
		switch ((ActionType)actiondata->actionType)
		{
		case ActionType::URI:
		{
			FSPDF_ACTIONDATA_URI *fsUrlData = (FSPDF_ACTIONDATA_URI *)actiondata->actionData;
			FSCRT_BStr_Clear(&(fsUrlData->url));
		}
		default:
			break;
		}
		FSCRT_Memory_Free((FS_LPVOID)actiondata->actionData);
	}
	FSCRT_Memory_Free((FS_LPVOID)actiondata);
}