#pragma once

#ifndef _PDFBASE_H_
#define _PDFBASE_H_

#include "Crt.h"

using namespace Platform;

namespace FSDKDLL
{
	namespace FSDK
	{
		/** @brief Saving Flags, used for saving document. */
		public enum class SaveAsFlag
		{
			Incremental = FSPDF_SAVEFLAG_INCREMENTAL,		/**< @brief Save the document incrementally. */
			NoOriginal = FSPDF_SAVEFLAG_NOORIGINAL,		/**< @brief Save the document without the original one. */
			RemoveSecurity = FSPDF_SAVEFLAG_REMOVESECURITY,	/**< @brief Save the document with removing security. */
			ObjectStream = FSPDF_SAVEFLAG_OBJECTSTREAM		/**< @brief Save the document by using object stream creator. */
		};

		public enum class searchFlag
		{
			/** @brief	Whether is it a matching case. */

			Matchcase = FSCRT_SEARCHFLAG_MATCHCASE,

			/** @brief	Whether is it a matching whole word. */

			MatchWholeWord = FSCRT_SEARCHFLAG_MATCHWHOLEWORD,

			/** @brief	Whether matching consecutively (for example, "CC" will match twice in "CCC").*/

			Conseutive = FSCRT_SEARCHFLAG_CONSECUTIVE,
		};

		/** @brief Page Rotation Flags */
		public enum class PageRotation
		{
			Degree_0 = FSCRT_PAGEROTATION_0,		/**< @brief	No rotation. */
			Degree_90 = FSCRT_PAGEROTATION_90,		/**< @brief	Rotate 90 degrees in clockwise direction. */
			Degree_180 = FSCRT_PAGEROTATION_180,		/**< @brief	Rotate 180 degrees in clockwise direction. */
			Degree_270 = FSCRT_PAGEROTATION_270,		/**< @brief	Rotate 270 degrees in clockwise direction. */
		};

		/** @brief Page Parse Flags */
		public enum class PageParseFlag
		{
			Normal = 0,		/**< @brief	PDF page parsing flag for normal mode. */
			TextOnly = 1	/**< @brief	PDF page parsing flag for text-only mode. */
		};

		/** @brief Dib format Flags. */
		public enum class PixelFormat
		{
			BGRx = 2			/**< @brief 4 bytes per pixel, byte order: Blue, Green, Red, not used. */
		};

		/** @brief Class to present the dib data created from Foxit PDF SDK.*/
		public ref class PixelSource sealed
		{
		public:
			/** @brief Default constructor function */
			PixelSource() {}

			property PixelFormat	Format;			/**< @brief The dib format defined with enum Foxit::General::PixelFormat. */
			property IBuffer^		PixelBuffer;	/**< @brief The dib pixel source buffer. */
			property int32			Width;			/**< @brief The width of dib. */
			property int32			Height;			/**< @brief The height of dib. */
		};

		/** @brief Default Appearance Flags */
		public enum class DefaultAppearanceFlag
		{
			Font = FSPDF_DEFAULTAPPEARANCE_FONT,			/**< @brief	The font flag of default appearance. */
			TextColor = FSPDF_DEFAULTAPPEARANCE_TEXTCOLOR,	/**< @brief	The text color flag of default appearance. */
			TextMatrix = FSPDF_DEFAULTAPPEARANCE_TEXTMATRIX	/**< @brief	The text matrix flag of default appearance flag. */
		};

		/** @brief	Class for default appearance.It may be used for form and annotation. */
		public ref class DefaultAppearance sealed
		{
		public:
			DefaultAppearance() {}
			/**
			* @brief	Flags of default appearance.
			*
			* @details	It can be one or a combination of the following flags: <br>
			*			<ul>
			*			<li>DemoRTSDK::Foxit::DefaultAppearanceFlag::Font</li>
			*			<li>DemoRTSDK::Foxit::DefaultAppearanceFlag::TextColor</li>
			*			<li>DemoRTSDK::Foxit::DefaultAppearanceFlag::TextMatrix</li>
			*			</ul>
			*/
			property uint32			flags;
			/** @brief	Reserved. */
			property uint32			reserved;
			/**
			* @brief	Font handle of default appearance.
			*
			* @details	Meaningful only when DemoRTSDK::Foxit::DefaultAppearance::flags contains DemoRTSDK::Foxit::DefaultAppearanceFlag::Font.
			*/
			property FontHandle		font;
			/**
			* @brief	Font size of default appearance. This should not be negative.
			*
			* @details	Meaningful only when DemoRTSDK::Foxit::DefaultAppearance::flags contains DemoRTSDK::Foxit::DefaultAppearanceFlag::Font.
			*/
			property float32		fontSize;
			/**
			* @brief	Text color of default appearance. Format: 0xAARRGGBB.
			*
			* @details	Meaningful only when DemoRTSDK::Foxit::DefaultAppearance::flags contains DemoRTSDK::Foxit::DefaultAppearanceFlag::TextColor.
			*/
			property uint32			textColor;
			/**
			* @brief	(Reserved, not support currently)Text matrix of default appearance.
			*
			* @details	Meaningful only when DemoRTSDK::Foxit::DefaultAppearance::flags contains DemoRTSDK::Foxit::DefaultAppearanceFlag::TextMatrix.
			*/
			property Matrix^		textMatrix;
		};

		/** @brief Action Types Flags */
		public enum class ActionType
		{
			Unsupport = FSPDF_ACTION_UNSUPPORTED,	/**< @brief Unsupported action type. */
			Goto = FSPDF_ACTION_GOTO,			/**< @brief Go to a destination within current document. */
			RemoteGoto = FSPDF_ACTION_REMOTEGOTO,		/**< @brief Go to a destination within another document. */
			EmbeddedGoto = FSPDF_ACTION_EMBEDDEDGOTO,	/**< @brief Go to a destination within embedded another document. */
			URI = FSPDF_ACTION_URI,			/**< @brief Universal Resource Identifier, including web pages and other Internet based resources. */
			Launch = FSPDF_ACTION_LAUNCH,			/**< @brief Launch an application or open a file. */
			JavaScript = FSPDF_ACTION_JAVASCRIPT,		/**< @brief Run a javascript. */
			Hide = FSPDF_ACTION_HIDE,			/**< @brief Hide or show one or more annotations. */
			Named = FSPDF_ACTION_NAMED,			/**< @brief Run a named action. */
			SubmitForm = FSPDF_ACTION_SUBMITFORM,		/**< @brief Transmit the names and values of selected interactive form fields. */
			ResetFrom = FSPDF_ACTION_RESETFORM,		/**< @brief Reset selected interactive form fields to their default values. */
			ImportData = FSPDF_ACTION_IMPORTDATA,		/**< @brief Import Forms Data Format (FDF) data into the document's interactive form. */
			Rendition = FSPDF_ACTION_RENDITION		/**< @brief Play a multimedia.*/
		};

		/**
		* @brief Annotation Types Flags
		* @note This only wraps part of annotations type defined in Foxit PDF SDK(C API).
		*/
		public enum class AnnotType
		{
			Text = 1,		/**< @brief Text annotation. */
			FreeText = 2,	/**< @brief FreeText annotation. */
			Link = 3,		/**< @brief Link annotation. */
			Line = 4,		/**< @brief Line annotation. */
			Polygon = 5,	/**< @brief Polygon annotation. */
			Square = 6,		/**< @brief Square annotation. */
			UnderLine = 7	/**< @brief Underline annotation. */
		};

		/**
		* @brief Class for URI Ation Data
		* @note This is a wrapper of FSPDF_ACTIONDATA_URI in Foxit PDF SDK(C API).
		*/
		public ref class ActionDataURI sealed
		{
		public:
			ActionDataURI() {}
			/** @brief URI address. */
			property String^			url;
			/** @brief Whether to track the mouse position when the URI is resolved. */
			property Boolean			mapPosition;
		};
		//////////////////////////////////////////////////////////////////////////////////////////////////////

		/**
		* @brief Class for PDF Base module.
		* @note This class is a wrapper for some methods defined in fpdf_base_r.h of Foxit PDF SDK(C API).
		*/
		public ref class PDFBase sealed
		{
		public:
			/**
			* @brief	Initialize PDF module.
			*
			* @details	This should be called before using any functions related with PDF.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_PDFModule_Initialize in Foxit PDF SDK(C API).
			*/
			static Boolean InitializeModule();

			/**
			* @brief	Release PDF module.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSCRT_PDFModule_Finalize in Foxit PDF SDK(C API).
			*/
			static Boolean ReleaseModule(); //FSCRT_PDFModule_Finalize

			/**
			* @brief	Create an action handle based on given action type and action data
			*
			* @param[in] type	An ActionType object.
			* @param[in] data	The action data.
			*					According to parameter <i>type</i>, different members of the data are used:
			*					(In this component only wraps URI action data.)
			*					<ul>
			*					<li>::ActionType::URI:				::Annot::ActionDataURI</li>
			*					</ul>
			*
			* @return	An action handle.
			*			When failed, please call FSDKDLL::FSDK::Crt::GetLastError to get the error status.
			*
			* @note		1. This function is a wrapper of FSPDF_ActionData_Init in Foxit PDF SDK(C API).
			*			2. Created action handle must be released by calling DemoRTSDK::Foxit::ReleaseActionData.
			*			3. Currently, in this component, it only wraps URI action data of Foxit PDF SDK(C API).
			*			So, if get error status of ErrorStatus::Unsupported, it may means input type is not ActionType::URI.
			*/
			static ActionHandle CreateActionData(ActionType type, Object^ data);

			/**
			* @brief	Release action handle.
			*
			* @param[in] action		An action handle.
			*
			* @note		This function is a wrapper of FSPDF_ActionData_Clear in Foxit PDF SDK(C API).
			*/
			static void ReleaseActionData(ActionHandle action);
		};
	}
}

#endif //_PDFBASE_H_
