#pragma once

#include "pch.h"
#include "PDFPage.h"

using namespace Platform;
using namespace concurrency;
using namespace FSDKDLL::FSDK;

using namespace Platform::Collections;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Foundation;
using namespace Windows::Graphics::Imaging;
using namespace concurrency;

////////////////////////////////////////////
// Class FSDKDLL::FSDK::PDFPage

Boolean PDFPage::Close()
{
	if (0 == m_pdfPage.pointer)
		return false;
	FS_RESULT ret = FSPDF_Page_Clear((FSCRT_PAGE)(m_pdfPage.pointer));

	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		return true;
	}
	else
		return false;
}

Boolean PDFPage::Parse(PageParseFlag flag)
{
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSCRT_PROGRESS progressParse = NULL;
	FS_RESULT ret = FSPDF_Page_StartParse((FSCRT_PAGE)m_pdfPage.pointer, (int32)flag, &progressParse);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_FINISHED == ret)
		return true;
	else if (FSCRT_ERRCODE_SUCCESS != ret)
		return false;

	//Continue parsing page.
	//If want to do progressive parsing, please use the second parameter of FSCRT_Progress_Continue.
	//See FSCRT_Progress_Continue for more details.
	ret = FSCRT_Progress_Continue(progressParse, NULL);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	FSCRT_Progress_Release(progressParse);
	return true;
}

FS_RESULT PDFPage::GetBitmapData(FSCRT_BITMAP bmp, PixelSource^ dib)
{
	void *lpBmpBuf = 0;
	FS_RESULT ret = FSCRT_Bitmap_GetLineBuffer(bmp, 0, &lpBmpBuf);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return ret;
	}
	unsigned int width = 0, height = 0;
	ret = FSCRT_Bitmap_GetSize(bmp, (FS_INT32 *)(&width), (FS_INT32 *)(&height));
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return ret;
	}
	unsigned int stride = 0;
	ret = FSCRT_Bitmap_GetLineStride(bmp, (FS_INT32 *)(&stride));
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return ret;
	}
	unsigned int size = stride * height;
	FSCRT_BITMAPINFO bitmapInfo;
	FS_DWORD bmp_infosize;
	FSCRT_Bitmap_GetBitmapInfo(bmp, NULL, &bmp_infosize);
	FSCRT_Bitmap_GetBitmapInfo(bmp, &bitmapInfo, &bmp_infosize);
	Array<unsigned char, 1>^ buffer = ref new Array<unsigned char, 1>(size);
	memcpy(buffer->Data, lpBmpBuf, size);
	DataWriter ^writer = ref new DataWriter();
	writer->WriteBytes(buffer);
	dib->Format = PixelFormat::BGRx;
	dib->PixelBuffer = writer->DetachBuffer();
	if (!dib->PixelBuffer)
	{
		ret = FSCRT_ERRCODE_ERROR;
	}

	return ret;
}

FS_RESULT PDFPage::PDFPageToBitmap(FSCRT_PAGE page, int bmpWidth, int bmpHeight, int iStartX,
	int iStartY, int iSizeX, int iSizeY, int iRotation, FSCRT_BITMAP *renderBmp)
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	//Get a bitmap handler to hold bitmap data from rendering progress.
	ret = FSCRT_Bitmap_Create((FS_INT32)bmpWidth, (FS_INT32)bmpHeight, FSCRT_BITMAPFORMAT_32BPP_RGBx, NULL, 0, renderBmp);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return ret;
	}

	//Set rect area and fill the color of bitmap.
	FSCRT_RECT rect = { 0, 0, (FS_INT32)bmpWidth, (FS_INT32)bmpHeight };
	ret = FSCRT_Bitmap_FillRect(*renderBmp, FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff), &rect);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Get the page's matrix.
	FSCRT_MATRIX mt;
	ret = FSPDF_Page_GetMatrix(page, iStartX, iStartY, iSizeX, iSizeY, iRotation, &mt);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Create a renderer based on a given bitmap, and page will be rendered to this bitmap.
	FSCRT_RENDERER renderer;
	ret = FSCRT_Renderer_CreateOnBitmap(*renderBmp, &renderer);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Create a render context used for rendering page.
	FSPDF_RENDERCONTEXT rendercontext;
	ret = FSPDF_RenderContext_Create(&rendercontext);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Renderer_Release(renderer);
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Please refer to comment of FSPDF_RenderContext_SetFlags and FSPDF_RENDERCONTEXTFLAG_XXX to know more about the flag used for rendercontext.
	ret = FSPDF_RenderContext_SetFlags(rendercontext, FSPDF_RENDERCONTEXTFLAG_ANNOT);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		//Release render
		FSCRT_Renderer_Release(renderer);
		FSPDF_RenderContext_Release(rendercontext);
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Set the matrix of the given render context.
	ret = FSPDF_RenderContext_SetMatrix(rendercontext, &mt);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Renderer_Release(renderer);
		FSPDF_RenderContext_Release(rendercontext);
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Start to render page.
	FSCRT_PROGRESS renderProgress = NULL;
	ret = FSPDF_RenderContext_StartPage(rendercontext, renderer, page, FSPDF_PAGERENDERFLAG_NORMAL, &renderProgress);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Renderer_Release(renderer);
		FSPDF_RenderContext_Release(rendercontext);
		FSCRT_Bitmap_Release(*renderBmp);
		return ret;
	}

	//Continue render progress.
	//If want to do progressive rendering, please use the second parameter of FSCRT_Progress_Continue.
	//See FSCRT_Progress_Continue for more details.
	ret = FSCRT_Progress_Continue(renderProgress, NULL);

	FSCRT_Progress_Release(renderProgress);
	FSCRT_Renderer_Release(renderer);
	FSPDF_RenderContext_Release(rendercontext);
	if (FSCRT_ERRCODE_FINISHED == ret)
		ret = FSCRT_ERRCODE_SUCCESS;
	else
		FSCRT_Bitmap_Release(*renderBmp);

	return ret;
}

Boolean PDFPage::GetRenderBitmapData(PixelSource^ pxsrc, PageHandle hPage, int iStartX, int iStartY, int iSizeX, int iSizeY, int iRotation)
{
	FSCRT_BITMAP renderBmp = NULL;
	FSCRT_PAGE pdfPage = (FSCRT_PAGE)hPage.pointer;
	//Render page to SDK bitmap.
	FS_RESULT ret = PDFPageToBitmap(pdfPage, pxsrc->Width, pxsrc->Height, iStartX, iStartY, iSizeX, iSizeY, iRotation, &renderBmp);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		return false;
	}

	//Get data of bitmap.
	ret = GetBitmapData(renderBmp, pxsrc);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	FSCRT_Bitmap_Release(renderBmp);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		return false;
	}
	else
	{
		return true;
	}
}

IAsyncOperation<IRandomAccessStreamWithContentType^>^ PDFPage::RenderPageAsync(PixelSource^ pxsrc, int32 iStartX, int32 iStartY,
	int32 iSizeX, int32 iSizeY, PageRotation rotation)
{
	return create_async([=]()->task < IRandomAccessStreamWithContentType^ > {

		bool ret = GetRenderBitmapData(pxsrc, m_pdfPage, iStartX, iStartY, iSizeX, iSizeY, (int32)rotation);
		if (true != ret)
		{
			return create_task([]()->IRandomAccessStreamWithContentType^ {return nullptr; });
		}
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_SUCCESS, NULL);
		InMemoryRandomAccessStream^ _stream = ref new InMemoryRandomAccessStream();
		return task<BitmapEncoder^>(BitmapEncoder::CreateAsync(BitmapEncoder::BmpEncoderId, _stream)).then([=](BitmapEncoder^ encoder)->task < IRandomAccessStreamWithContentType^ > {
			DataReader^ reader = DataReader::FromBuffer(pxsrc->PixelBuffer);
			unsigned int bmp_length = pxsrc->PixelBuffer->Length;
			Array<unsigned char, 1>^ buffer = ref new Array<unsigned char, 1>(pxsrc->PixelBuffer->Length);
			if (!buffer)
			{
				FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_OUTOFMEMORY, NULL);
				return create_task([]()->IRandomAccessStreamWithContentType^ {return nullptr; });
			}
			reader->ReadBytes(buffer);

			encoder->SetPixelData(BitmapPixelFormat::Rgba8, BitmapAlphaMode::Straight, pxsrc->Width, pxsrc->Height, 96.0, 96.0, buffer);
			return task<void>(encoder->FlushAsync()).then([=]()->task < IRandomAccessStreamWithContentType^ > {
				RandomAccessStreamReference^ streamReference = RandomAccessStreamReference::CreateFromStream(_stream);
				return task<IRandomAccessStreamWithContentType^>(streamReference->OpenReadAsync()).then([=](IRandomAccessStreamWithContentType^ ad)->IRandomAccessStreamWithContentType^ {
					return ad;
				});
			});
		});
	});
}

float32 PDFPage::GetWidth()
{
	float fWidth = 0.0f;
	if (m_pdfPage.pointer < 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return fWidth;
	}
	float32 fHeight;
	FS_RESULT ret = FSPDF_Page_GetSize((FSCRT_PAGE)(m_pdfPage.pointer), &fWidth, &fHeight);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return fWidth;
}

float32 PDFPage::GetHeight()
{
	float32 fHeight = 0.0f;
	if (m_pdfPage.pointer < 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return fHeight;
	}
	float32 fWidth;
	FS_RESULT ret = FSPDF_Page_GetSize((FSCRT_PAGE)(m_pdfPage.pointer), &fWidth, &fHeight);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return fHeight;
}

Boolean PDFPage::LoadAnnots()
{
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FS_RESULT ret = FSPDF_Page_LoadAnnots((FSCRT_PAGE)m_pdfPage.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (ret == FSCRT_ERRCODE_SUCCESS)
		return true;

	return false;
}

Boolean PDFPage::UnLoadAnnots()
{
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_Page_UnloadAnnots((FSCRT_PAGE)m_pdfPage.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

String^ AnnotTypeToString(AnnotType annottype)
{
	String^ typestr = ref new String();
	switch (annottype)
	{
	case AnnotType::Text:
	{
		typestr = FSPDF_ANNOTTYPE_TEXT;
		break;
	}
	case AnnotType::FreeText:
	{
		typestr = FSPDF_ANNOTTYPE_FREETEXT;
		break;
	}
	case AnnotType::Link:
	{
		typestr = FSPDF_ANNOTTYPE_LINK;
		break;
	}
	case AnnotType::Line:
	{
		typestr = FSPDF_ANNOTTYPE_LINE;
		break;
	}
	case AnnotType::Polygon:
	{
		typestr = FSPDF_ANNOTTYPE_POLYGON;
		break;
	}
	case AnnotType::Square:
	{
		typestr = FSPDF_ANNOTTYPE_SQUARE;
		break;
	}
	case AnnotType::UnderLine:
	{
		typestr = FSPDF_ANNOTTYPE_UNDERLINE;
		break;
	}
	default:
	{
		return nullptr;
	}
	}

	return typestr;
}

String^ AnnotFilterToString(IVector<AnnotType>^ filter)
{
	if (nullptr == filter)
		return nullptr;

	int filterSize = filter->Size;
	String^ str = ref new String();
	for (int i = 0; i < filterSize; i++)
	{
		if (0 != i)
			str += ", ";
		str += AnnotTypeToString(filter->GetAt(i));
	}
	return str;
}

int32 PDFPage::GetAnnotsCount(IVector<AnnotType>^ filter)
{
	FS_INT32 count = 0;
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return count;
	}

	String^ str = AnnotFilterToString(filter);
	auto bsfilter = ConvertToFoxitString(str);

	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	if ((bsfilter->len < 1) || (bsfilter->str == NULL))
	{
		ret = FSPDF_Annot_GetCount((FSCRT_PAGE)m_pdfPage.pointer, NULL, &count);
	}
	else
	{
		ret = FSPDF_Annot_GetCount((FSCRT_PAGE)m_pdfPage.pointer, &*bsfilter, &count);
	}
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return count;
}

PDFAnnot^ PDFPage::GetAnnot(IVector<AnnotType>^ filter, int32 index)
{
	PDFAnnot^ pdfAnnot = nullptr;

	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return pdfAnnot;
	}

	String^ str = AnnotFilterToString(filter);
	auto bsfilter = ConvertToFoxitString(str);
	FSCRT_ANNOT annot = NULL;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	if ((bsfilter->len < 1) || (NULL == bsfilter->str))
	{
		ret = FSPDF_Annot_Get((FSCRT_PAGE)m_pdfPage.pointer, NULL, index, &annot);
	}
	else
	{
		ret = FSPDF_Annot_Get((FSCRT_PAGE)m_pdfPage.pointer, &*bsfilter, index, &annot);
	}
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		AnnotHandle annotGet;
		annotGet.pointer = (int64)annot;

		pdfAnnot = ref new PDFAnnot(annotGet);
	}
		
	return pdfAnnot;
}

PDFAnnot^ PDFPage::AddAnnot(RectF^ rectf, AnnotType annottype, IVector<AnnotType>^ filter, int32 index)
{
	PDFAnnot^ pdfAnnot = nullptr;

	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return pdfAnnot;
	}

	//get char of annot type
	String^ typestr = AnnotTypeToString(annottype);
	if (nullptr == typestr || typestr->IsEmpty())
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return pdfAnnot;
	}
	auto bsAnnotType = ConvertToFoxitString(typestr);

	//get char of filter
	String^ sFilterStr = AnnotFilterToString(filter);
	auto bsFilter = ConvertToFoxitString(sFilterStr);

	//get rect from param
	FSCRT_RECTF fsRectF = { 0 };
	if (rectf != nullptr)
	{
		fsRectF.bottom = rectf->Bottom;
		fsRectF.left = rectf->Left;
		fsRectF.right = rectf->Right;
		fsRectF.top = rectf->Top;
	}

	//Add annot
	FSCRT_ANNOT pdfannot = NULL;
	FS_RESULT ret = FSPDF_Annot_Add((FSCRT_PAGE)(m_pdfPage.pointer), &fsRectF, &*bsAnnotType, &*bsFilter, index, &pdfannot);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		AnnotHandle annotAdd;
		annotAdd.pointer = (int64)pdfannot;

		pdfAnnot = ref new PDFAnnot(annotAdd);
	}
	return pdfAnnot;
}

PDFText^ PDFPage::LoadTextPage()
{
	PDFText^ pdfText = nullptr;
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return pdfText;
	}
	FSPDF_TEXTPAGE pdfTextpage = NULL;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSPDF_TextPage_Load((FSCRT_PAGE)m_pdfPage.pointer, &pdfTextpage);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		TextPageHandle textPageHandle;
		textPageHandle.pointer = (int64)pdfTextpage;
		pdfText = ref new PDFText(textPageHandle);
	}
	return pdfText;
}

Boolean PDFPage::TransformPageMatrixToPointF(int32 pageX, int32 pageY, int32 pageWidth, int32 pageHeight, int32 rotation, PointF^ pointF)
{
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FSCRT_MATRIX pageMatrix, deviceMatrix;
	FS_RESULT ret = FSPDF_Page_GetMatrix((FSCRT_PAGE)m_pdfPage.pointer, pageX, pageY, pageWidth, pageHeight, rotation, &pageMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		return false;
	}

	ret = FSCRT_Matrix_GetReverse(&pageMatrix, &deviceMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		return false;
	}

	FS_FLOAT x = pointF->X, y = pointF->Y;

	ret = FSCRT_Matrix_TransformPointF(&deviceMatrix, &x, &y);

	pointF->X = x;
	pointF->Y = y;

	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return true;
}

RectF^ PDFPage::TransformPageMatrixToRectF(int32 pageX, int32 pageY, int32 pageWidth, int32 pageHeight, int32 rotation, RectF^ rectF)
{
	if (0 == m_pdfPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return nullptr;
	}

	FSCRT_MATRIX matrix;
	FS_RESULT ret = FSPDF_Page_GetMatrix((FSCRT_PAGE)m_pdfPage.pointer, pageX, pageY, pageWidth, pageHeight, rotation, &matrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		if (ret != FSCRT_ERRCODE_SUCCESS)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
			return nullptr;
		}
	}

	FSCRT_RECTF sdkRectF;
	sdkRectF.left = rectF->Left;
	sdkRectF.right = rectF->Right;
	sdkRectF.top = rectF->Top;
	sdkRectF.bottom = rectF->Bottom;

	ret = FSCRT_Matrix_TransformRectF(&matrix, &sdkRectF);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	RectF^ rectResult = ref new RectF;

	rectResult->Left = sdkRectF.left;
	rectResult->Top = sdkRectF.top;
	rectResult->Right = sdkRectF.right;
	rectResult->Bottom = sdkRectF.bottom;

	return rectResult;
}
