#include "pch.h"
#include "PDFText.h"

using namespace Platform;
using namespace Platform::Collections;
using namespace FSDKDLL::FSDK;
using namespace concurrency;

////////////////////////////////////////
//Class FSDKDLL::FSDK::PDFText

Boolean PDFText::ReleaseTextPage()
{
	if (0 == m_pdfTextPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSPDF_TextPage_Release((FSPDF_TEXTPAGE)m_pdfTextPage.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		return true;
	else
		return false;
}

int32 PDFText::CountChars()
{
	FS_INT32 iCharCount = -1;
	if (0 == m_pdfTextPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return iCharCount;
	}
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSPDF_TextPage_CountChars((FSPDF_TEXTPAGE)m_pdfTextPage.pointer, &iCharCount);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return iCharCount;
}

int32 PDFText::GetCharIndexAtPos(PointF^ pointF, float tolerance)
{
	if (m_pdfTextPage.pointer == 0 || pointF == nullptr)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return -1;
	}

	FS_INT32 iCharIndex = -1;
	FS_RESULT ret = FSPDF_TextPage_GetCharIndexAtPos((FSPDF_TEXTPAGE)m_pdfTextPage.pointer, pointF->X, pointF->Y, tolerance, &iCharIndex);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return iCharIndex;
}

int32 PDFText::GetStartCharIndex(float pageX, int charIndex, int startCharIndex, int endCharIndex)
{
	if (m_pdfTextPage.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return -1;
	}

	int32 index = -1;
	FSPDF_TEXTPAGE_CHARINFO info;
	FS_RESULT ret = FSPDF_TextPage_GetCharInfo((FSPDF_TEXTPAGE)m_pdfTextPage.pointer, charIndex, &info);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	float positionTmp = 0;

	if (endCharIndex - startCharIndex >= 0)
	{
		if (pageX > (info.bbox.left + info.bbox.right) / 2)
			index = charIndex + 1;
		else
			index = charIndex;
	}
	else
	{
		if (pageX > (info.bbox.left + info.bbox.right) / 2)
			index = charIndex;
		else
			index = charIndex - 1;
	}
	return index;
}

Boolean PDFText::SelectTextByRange(int32 start, int32 count)
{
	if (m_pdfSelectionText.pointer != 0)
	{
		ReleaseTextSelection();
	}
	TextSelectionHandle textSelect;
	textSelect.pointer = 0;
	if (0 == m_pdfTextPage.pointer)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FSPDF_TEXTSELECTION pdfTextselect = NULL;
	FS_RESULT ret = FSPDF_TextPage_SelectByRange((FSPDF_TEXTPAGE)m_pdfTextPage.pointer, start, count, &pdfTextselect);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		textSelect.pointer = (int64)pdfTextselect;

	m_pdfSelectionText = textSelect;

	return true;
}

String^ PDFText::GetTextSelectionChars()
{
	if (0 == m_pdfTextPage.pointer || m_pdfSelectionText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return "";
	}
	FSCRT_BSTR textSelectionString;
	FSCRT_BStr_Init(&textSelectionString);
	FS_RESULT ret = FSPDF_TextSelection_GetChars((FSPDF_TEXTSELECTION)m_pdfSelectionText.pointer, &textSelectionString);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		FS_WORD* buffer = NULL;
		FS_DWORD bufLen = 0;
		ret = FSCRT_UTF8_ToUTF16LE(&textSelectionString, buffer, &bufLen);
		if (FSCRT_ERRCODE_SUCCESS != ret || bufLen == 0)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
			return "";
		}
		buffer = new FS_WORD[bufLen + 1];
		memset(buffer, 0, bufLen + 1);
		ret = FSCRT_UTF8_ToUTF16LE(&textSelectionString, buffer, &bufLen);
		buffer[bufLen] = 0;
		String^ string = ref new String((const wchar_t*)buffer);
		return string;
	}

	return "";
}

Boolean PDFText::ReleaseTextSelection()
{
	FSPDF_TEXTSELECTION textSelection = (FSPDF_TEXTSELECTION)m_pdfSelectionText.pointer;
	if (!textSelection)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSPDF_TextSelection_Release((FSPDF_TEXTSELECTION)m_pdfSelectionText.pointer);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		TextSelectionHandle textSelectionHandle;
		m_pdfSelectionText = textSelectionHandle;
		return true;
	}
	else
		return false;
}

int32 PDFText::CountSelectPieces()
{
	FS_INT32 iPieceCount = 0;
	if (m_pdfSelectionText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return iPieceCount;
	}
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSPDF_TextSelection_CountPieces((FSPDF_TEXTSELECTION)m_pdfSelectionText.pointer, &iPieceCount);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	return iPieceCount;
}

RectF^ PDFText::GetSelectPieceRect(int32 pieceIndex)
{
	if (m_pdfSelectionText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return nullptr;
	}
	FSCRT_RECTF pdfRectF;
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	ret = FSPDF_TextSelection_GetPieceRect((FSPDF_TEXTSELECTION)m_pdfSelectionText.pointer, pieceIndex, &pdfRectF);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		RectF^ rectF = ref new RectF();
		rectF->Left = pdfRectF.left;
		rectF->Top = pdfRectF.top;
		rectF->Right = pdfRectF.right;
		rectF->Bottom = pdfRectF.bottom;
		return rectF;
	}
	else
		return nullptr;
}

IAsyncOperation<Boolean>^ PDFText::ExportToTxtFile(StorageFile^ savefile, File^ sdkfile)
{
	auto op = create_async([=]()->task<Boolean> {
		if (nullptr == savefile || nullptr == sdkfile || m_pdfTextPage.pointer == 0)
		{
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
			return create_task([]()->bool {return false; });
		}
		
		return concurrency::create_task([=]()
		{
			IObservableVector<Platform::Object^>^ fileBuffer = nullptr;
			FS_RESULT ret = FSPDF_TextPage_ExportToFile((FSPDF_TEXTPAGE)m_pdfTextPage.pointer, (FSCRT_FILE)sdkfile->GetFileHandle().pointer);
			FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
			
			if (FSCRT_ERRCODE_SUCCESS == ret)
			{
				fileBuffer = ref new Vector<Object^>();
				fileBuffer = sdkfile->GetFileBuffer();
			}
			std::shared_ptr<DataWriter^> _spDataWriter = std::make_shared<DataWriter^>(nullptr);
			if (nullptr == sdkfile || nullptr == fileBuffer)
			{
				return create_task([]()->IOutputStream^ {return nullptr; });
			}
			auto OpenOp = savefile->OpenAsync(FileAccessMode::ReadWrite);
			return create_task(OpenOp).then([=](IRandomAccessStream^ writeStream)
			{
				writeStream->Seek(0);
				*_spDataWriter = ref new DataWriter(writeStream);
				auto size = fileBuffer->Size;
				for (unsigned int i = 0; i < size; i++)
				{
					Platform::Array<unsigned char>^ fileContent = safe_cast<Platform::Array<unsigned char>^>(fileBuffer->GetAt(i));
					(*_spDataWriter)->WriteBytes(fileContent);
				}
				return (*_spDataWriter)->StoreAsync();
			}).then([=](unsigned int bytesStored) {
				return (*_spDataWriter)->FlushAsync();
			}).then([=](bool flushSucceeded) {
				fileBuffer->Clear();
				return (*_spDataWriter)->DetachStream();
			});
		})
			.then([=](concurrency::task<IOutputStream^> t)
		{
			IOutputStream^ outputStream = t.get();
			if (nullptr == outputStream)
			{
				FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_ERROR, NULL);
				return false;
			}
			else
			{
				FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_SUCCESS, NULL);
				return true;
			}
		});
	});
	return op;
}

Boolean PDFText::StartSearch(Platform::String^ searchPattern, FS_DWORD flags, FS_INT32 startIndex)
{
	if (m_pdfSearchText.pointer != 0)
	{
		ReleaseTextSearch();
	}

	TextSearchHandle searchHandle;
	if (m_pdfTextPage.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FSCRT_BSTR strSearchPattern;
	FSCRT_BStr_Init(&strSearchPattern);

	FS_RESULT ret = FSCRT_UTF8_FromUTF16LE(&strSearchPattern, (FS_WORD*)(LPCWSTR)(searchPattern->Data()), (FS_DWORD)searchPattern->Length());
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);
		FSCRT_BStr_Clear(&strSearchPattern);
		return false;
	}

	ret = FSPDF_TextPage_StartSearch((FSPDF_TEXTPAGE)m_pdfTextPage.pointer, &strSearchPattern, flags, startIndex, (FSPDF_TEXTSEARCH*)(&(searchHandle.pointer)));
	FSCRT_BStr_Clear(&strSearchPattern);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	m_pdfSearchText = searchHandle;
	return true;
}

Boolean PDFText::SearchFindNext()
{
	if (m_pdfTextPage.pointer == 0 || m_pdfSearchText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FS_BOOL isMatch = false;
	FS_RESULT ret = FSPDF_TextSearch_FindNext((FSPDF_TEXTSEARCH)m_pdfSearchText.pointer, &isMatch);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	return isMatch;
}

Boolean PDFText::SearchFindPrev()
{
	if (m_pdfTextPage.pointer == 0 || m_pdfSearchText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FS_BOOL isMatch;
	FS_RESULT ret = FSPDF_TextSearch_FindPrev((FSPDF_TEXTSEARCH)m_pdfSearchText.pointer, &isMatch);
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	return isMatch;
}

Boolean PDFText::ReleaseTextSearch()
{
	if (m_pdfTextPage.pointer == 0 || m_pdfSearchText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}

	FS_RESULT ret = FSPDF_TextSearch_Release((FSPDF_TEXTSEARCH)m_pdfSearchText.pointer);

	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		TextSearchHandle textSearch;
		m_pdfSearchText = textSearch;
		return true;
	}
	else
		return false;
}

Boolean PDFText::GetTextSelection()
{
	if (m_pdfSelectionText.pointer != 0)
	{
		ReleaseTextSelection();
	}

	TextSelectionHandle selectionHandle;
	if (m_pdfTextPage.pointer == 0 || m_pdfSearchText.pointer == 0)
	{
		FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)FSCRT_ERRCODE_PARAM, NULL);
		return false;
	}
	FS_RESULT ret = FSPDF_TextSearch_GetSelection((FSPDF_TEXTSEARCH)m_pdfSearchText.pointer, (FSPDF_TEXTSELECTION*)(&(selectionHandle.pointer)));
	FSCRT_Library_SetPrivateData(ERRORCODE_MODULE, (FS_LPVOID)(int)ret, NULL);

	m_pdfSelectionText = selectionHandle;

	return true;
}