#pragma once

#ifndef _PDFTEXT_H_
#define _PDFTEXT_H_

#include "PDFBase.h"

using namespace Platform;

namespace FSDKDLL
{
	namespace FSDK
	{
		/**
		* @brief Class for PDF text page module.
		* @note This class is a wrapper for some methods defined in fpdf_textpage_r.h of Foxit PDF SDK(C API).
		*/
		public ref class PDFText sealed
		{
		public:
			PDFText(TextPageHandle textPage)
			{
				m_pdfTextPage = textPage;
			}

			TextPageHandle GetTextPageHandle()
			{
				return m_pdfTextPage;
			}

			TextSearchHandle GetTextSearchHandle()
			{
				return m_pdfSearchText;
			}

			TextSelectionHandle GetTextSelectionHandle()
			{
				return m_pdfSelectionText;
			}
			/**
			* @brief Release all resources allocated for a text page handle.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextPage_Release in Foxit PDF SDK(C API).
			*/
			Boolean ReleaseTextPage();

			/**
			* @brief	Get count of characters in a page.
			*
			*
			* @return	The count of characters. If an error occurs, it will return -1.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextPage_CountChars in Foxit PDF SDK(C API).
			*/
			int32 CountChars();

			int32 GetCharIndexAtPos(PointF^ pointF, float tolerance);
			int32 GetStartCharIndex(float pageX, int charIndex, int startCharIndex, int endCharIndex);

			/**
			* @brief	Get a text selection handle by specific character range.
			*
			* @param[in]  start				A zero-based index of the start character.
			*								Range: from 0 to (charcount - 1).<i>charcount</i> is returned by function ::Text::CountChars..
			* @param[in]  count				Count of characters to be extracted. -1 means cover the whole characters in the page..
			*
			* @return	A text selection handle.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextPage_SelectByRange in Foxit PDF SDK(C API).
			*/
			Boolean SelectTextByRange(int32 start, int32 count);

			/**
			* @brief Release all resources allocated for a text selection handle.
			*
			* @param[in] textSelection			A text selection handle.
			*
			* @return	A boolean value: true means success, while false means failure.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextSelection_Release in Foxit PDF SDK(C API).
			*/
			Boolean ReleaseTextSelection();

			/**
			* @brief	Count the number of all rectangular areas of segments in a PDF text selected area.
			*
			* @param[in] textSelection	A text selection handle.
			*
			* @return	The count of characters. If an error occurs, it will return 0.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextPage_CountChars in Foxit PDF SDK(C API).
			*/
			int32 CountSelectPieces();

			/**
			* @brief	Get rectangular area of a specific segment, based on the result of the fucntion DemoRTSDK::Foxit::PDFText::CountSelectPieces.
			*
			* @param[in] textSelection	A text selection handle.
			* @param[in] pieceIndex		A zero-based index of the segment.
			*							Range: from 0 to (segmentcount - 1).<i>segmentcount</i> is returned by function ::Text::CountSelectPieces.
			*
			* @return	Reference to a ::RectF class that receives the 4 boundaries(left, top, right, bottom) of rectangle area.
			*			If there's any error, it will return nullptr.
			*			When failed, please call DemoRTSDK::Foxit::Base::GetLastError to get the error status.
			*
			* @note	This function is a wrapper of FSPDF_TextPage_CountChars in Foxit PDF SDK(C API).
			*/
			RectF^ GetSelectPieceRect(int32 pieceIndex);

			IAsyncOperation<Boolean>^ ExportToTxtFile(StorageFile^ savefile, File^ sdkfile);

			Boolean StartSearch(Platform::String^ searchPattern, FS_DWORD flags, FS_INT32 startIndex);

			Boolean SearchFindNext();

			Boolean SearchFindPrev();

			Boolean ReleaseTextSearch();

			Boolean GetTextSelection();

			String^ PDFText::GetTextSelectionChars();
		private:
			property TextPageHandle m_pdfTextPage;
			property TextSearchHandle m_pdfSearchText;
			property TextSelectionHandle m_pdfSelectionText;
		};
	}
}

#endif //_DEMORTPDFTEXT_H_