#include "pch.h"
#include "Util.h"
#include "Crt.h"

using namespace Platform;
using namespace concurrency;
using namespace FSDKDLL::FSDK;

using namespace Platform::Collections;
using namespace Windows::Storage;
using namespace Windows::Storage::Streams;
using namespace Windows::Storage::FileProperties;
using namespace Windows::Foundation;
using namespace Windows::Graphics::Imaging;

std::shared_ptr<FSCRT_BSTR> ConvertToFoxitString(Platform::String^ str)
{
	//shared_ptr is a smart pointer class. The first param in construct is the pointer, the second param is a dtor.
	std::shared_ptr<FSCRT_BSTR> _FSstr(new FSCRT_BSTR(), [](FSCRT_BSTR* param)
	{
		if (param)
			FSCRT_BStr_Clear(param);
	});

	FSCRT_BStr_Init(&*_FSstr);
	if (str != nullptr)
	{
		FSCRT_UTF8_FromUTF16LE(&*_FSstr, (FS_WORD*)str->Data(), str->Length());
	}

	return _FSstr;
}

CMy_File::CMy_File()
{
}

CMy_File::~CMy_File()
{
}

void CMy_File::InitFileHandle(CMy_FileReadWrite* pFileHandle)
{
	clientData = pFileHandle;
	Release = g_PrivateRelease;
	GetSize = g_PrivateGetSize;
	ReadBlock = g_PrivateReadBlock;
	WriteBlock = g_PrivateWriteBlock;
	Flush = g_PrivateFlush;
	Truncate = g_PrivateTruncate;
}

void CMy_File::ReleaseFileHandle()
{
	if (clientData)
		delete clientData;
	clientData = NULL;
	Release = NULL;
	GetSize = NULL;
	ReadBlock = NULL;
	WriteBlock = NULL;
	Flush = NULL;
	Truncate = NULL;
}

void CMy_File::g_PrivateRelease(FS_LPVOID clientData)
{
	CMy_FileReadWrite* pFileData = (CMy_FileReadWrite*)clientData;

	if (pFileData)
	{
		delete pFileData;
	}
	pFileData = NULL;
}

FS_DWORD CMy_File::g_PrivateGetSize(FS_LPVOID clientData)
{
	CMy_FileReadWrite* pFileData = (CMy_FileReadWrite*)clientData;
	return pFileData->GetSize();
}

FS_RESULT CMy_File::g_PrivateReadBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size)
{
	CMy_FileReadWrite* pFileData = (CMy_FileReadWrite*)clientData;

	size_t tSize = (size_t)size;

	if (!pFileData->ReadBlock(buffer, offset, tSize))
		return FSCRT_ERRCODE_ERROR;
	else
		return FSCRT_ERRCODE_SUCCESS;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CMy_File::g_PrivateWriteBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size)
{
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Change file pointer to offset
	CMy_FileReadWrite* pFileData = (CMy_FileReadWrite*)clientData;
	size_t tSize = (size_t)size;

	if (!pFileData->WriteBlock(buffer, offset, tSize))
		return FSCRT_ERRCODE_ERROR;
	else
		return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CMy_File::g_PrivateFlush(FS_LPVOID clientData)
{
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT CMy_File::g_PrivateTruncate(FS_LPVOID clientData, FS_DWORD size)
{
	return FSCRT_ERRCODE_SUCCESS;
}

/////////////////////////////////////////////////////////////////////
// Class CMy_FileReadWrite
CMy_FileReadWrite::CMy_FileReadWrite()
{
	m_pFile = nullptr;
}

CMy_FileReadWrite::~CMy_FileReadWrite()
{
	Release();
}

void CMy_FileReadWrite::SetSize(int nSize)
{
	m_iFileSize = nSize;
}

void CMy_FileReadWrite::CreateFileRead(Windows::Storage::StorageFile^ pFile, int nFileSize)
{
	m_pFile = pFile;
	m_iFileSize = nFileSize;
}

void CMy_FileReadWrite::Release()
{
	m_pFile = nullptr;
	if (m_Buffer && m_Buffer->Size > 0)
	{
		m_Buffer->Clear();
	}
}

int	CMy_FileReadWrite::GetSize()
{
	return m_iFileSize;
}

bool CMy_FileReadWrite::ReadBlock(void* buffer, int offset, size_t size)
{
	if (m_pFile == nullptr)
		return FALSE;

	bool _bIsReadOk = false;
	HANDLE processingHandle = CreateEventEx(
		NULL,
		NULL,
		false,
		EVENT_ALL_ACCESS
		);
	auto  OpenOp = m_pFile->OpenAsync(Windows::Storage::FileAccessMode::Read);
	OpenOp->Completed = ref new AsyncOperationCompletedHandler<IRandomAccessStream^>([buffer, offset, size, processingHandle, &_bIsReadOk](IAsyncOperation<IRandomAccessStream^>^ operation, Windows::Foundation::AsyncStatus status)
	{
		if (status == Windows::Foundation::AsyncStatus::Completed)
		{
			IRandomAccessStream^ readStream = operation->GetResults();
			readStream->Seek(offset);
			DataReader^ dataReader = ref new DataReader(readStream);
			auto LoadOp = dataReader->LoadAsync(size);
			LoadOp->Completed = ref new AsyncOperationCompletedHandler<unsigned int>([buffer, dataReader, processingHandle, &_bIsReadOk](IAsyncOperation<unsigned int>^ operation, Windows::Foundation::AsyncStatus status)
			{
				if (status == Windows::Foundation::AsyncStatus::Completed)
				{
					int numBytesLoaded = operation->GetResults();
					if (numBytesLoaded > 0)
					{
						Array<unsigned char, 1>^ fileContent = ref new Array<unsigned char, 1>(numBytesLoaded);
						dataReader->ReadBytes(fileContent);
						memcpy(buffer, fileContent->Data, numBytesLoaded);
						dataReader->DetachStream();
					}
					_bIsReadOk = true;
					SetEvent(processingHandle);
				}
				else
				{
					_bIsReadOk = false;
					OutputDebugString(L"LoadAsync status ERROR!!!!!\n");
					SetEvent(processingHandle);
				}
			});
		}
		else
		{
			_bIsReadOk = false;
			OutputDebugString(L"status ERROR!!!!!\n");
			SetEvent(processingHandle);
		}
	});
	WaitForSingleObjectEx(processingHandle, INFINITE, true);
	CloseHandle(processingHandle);
	return _bIsReadOk ? TRUE : FALSE;

}

bool CMy_FileReadWrite::WriteBlock(const void* buffer, int offset, size_t size)
{
	if (nullptr == m_Buffer)
	{
		m_Buffer = ref new Vector<Object^>();
	}

	Platform::Array<unsigned char>^ fileContent = ref new Platform::Array<unsigned char>((unsigned char*)buffer, size);
	m_Buffer->Append(fileContent);
	int size1 = m_Buffer->Size;
	return TRUE;
}

void CMy_FileReadWrite::ReleaseTempWriteBuffer()
{
	if (m_Buffer->Size > 0)
	{
		m_Buffer->Clear();
		m_Buffer = nullptr;
	}
}

Platform::Collections::Vector<Platform::Object^>^ CMy_FileReadWrite::GetFileBuffer()
{
	return m_Buffer;
}