#pragma once

#ifndef _UTIL_H_
#define _UTIL_H_
#include "pch.h"
#include <iostream>
#include <vector>
#include <ppltasks.h>

using namespace Platform;
using namespace Windows::Storage::Streams;

//Convert string from Platform::String to FSCRT_BSTR. Only for internal use.
std::shared_ptr<FSCRT_BSTR> ConvertToFoxitString(Platform::String^ str);

// Class CMy_FileReadWrite and CMy_File are used to implement the callback functions in FSCRT_FILEHANDLER.
// In this component , we use FSCRT_FILEHANDLER to create a FSCRT_FILE used for loading or saving PDF document.
// CMy_File is inherited from FSCRT_FILEHANDLER, but main implementation cods are in CMy_FileReadWrite.
class CMy_FileReadWrite
{
public:
	CMy_FileReadWrite();
	virtual ~CMy_FileReadWrite();
	void	CreateFileRead(Windows::Storage::StorageFile^ pFile, int nFileSize);
	void	Release();
	int		GetSize();
	void	SetSize(int nSize);
	bool	ReadBlock(void* buffer, int offset, size_t size);
	bool	WriteBlock(const void* buffer, int offset, size_t size);
	void	ReleaseTempWriteBuffer();

	Platform::Collections::Vector<Platform::Object^>^ GetFileBuffer();
private:
	Windows::Storage::StorageFile^						m_pFile;
	int													m_iFileSize;
	Platform::Collections::Vector<Platform::Object^>^	m_Buffer;
};

//Inherited class of FSCRT_FILEHANDLER
class CMy_File : public FSCRT_FILEHANDLER
{
public:
	CMy_File();
	~CMy_File();
	void InitFileHandle(CMy_FileReadWrite* pFileHandle);
	void ReleaseFileHandle();

	//Inherited callback funtions.
	static void g_PrivateRelease(FS_LPVOID clientData);
	static FS_DWORD g_PrivateGetSize(FS_LPVOID clientData);
	static FS_RESULT g_PrivateReadBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size);
	static FS_RESULT g_PrivateWriteBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size);
	static FS_RESULT g_PrivateFlush(FS_LPVOID clientData);
	static FS_RESULT g_PrivateTruncate(FS_LPVOID clientData, FS_DWORD size);
private:
};

#endif //_UTIL_H_
